package hu.uw.pallergabor.peeble;

import android.app.Activity;
import android.content.Context;
import android.content.ContentResolver;
import android.content.ContentValues;
import android.database.Cursor;
import android.location.LocationProvider;
import android.location.LocationManager;
import android.net.Uri;
import android.os.Bundle;
import android.view.View;
import android.widget.Button;
import android.widget.EditText;
import android.widget.Spinner;
import android.widget.SpinnerAdapter;
import android.widget.ArrayAdapter;
import android.widget.CheckBox;
import android.widget.Toast;
import java.util.List;

/**
  * Activity handling the configuration view.
  */
public class Config extends Activity {
// Default value of the sync address
	public static final String DEFAULTVALUE_SYNCADDRESS = "http://192.168.0.1:8088/peebleweb/SyncServer";
// Default value of the sync user
	public static final String DEFAULTVALUE_SYNCUSER = "testuser";
// Default value of the sync user
	public static final String DEFAULTVALUE_SYNCPASSWORD = "testpassword";
// Default value of the warning enabled field ("true"/"false")
    public static final String DEFAULTVALUE_WARNINGENABLED = "true";
// Default value of the warning timeout (in minutes)
	public static final int DEFAULTVALUE_WARNINGTIMEOUT = 60;
// Tracking accuracy in seconds
	public static final int DEFAULTVALUE_NOTIFICATIONINTERVAL = 10;
// Tracking accuracy in meters
	public static final int DEFAULTVALUE_NOTIFICATIONDISTANCE = 100;

	public static final String CONFIGVALUE_LOCPROVIDER = "locprovider";
	public static final String CONFIGVALUE_SYNCADDRESS = "syncaddress";
	public static final String CONFIGVALUE_SYNCUSER = "syncuser";
	public static final String CONFIGVALUE_SYNCPASSWORD = "syncpassword";
	public static final String CONFIGVALUE_WARNINGENABLED = "warningenabled";
	public static final String CONFIGVALUE_WARNINGTIMEOUT = "warningtimeout";
	public static final String CONFIGVALUE_NOTIFICATIONINTERVAL = "notificationinterval";
	public static final String CONFIGVALUE_NOTIFICATIONDISTANCE = "notificationdistance";

    @Override
    protected void onCreate(Bundle icicle) {
      super.onCreate(icicle);
      setContentView(R.layout.config);

// Setting up the spinner
      Spinner locselector = (Spinner)findViewById( R.id.locprovider_spinner );
	  String[] locProviderNames = getLocationProviderNames();
	  int selectedProviderIndex = -1;
	  String selectedProvider = getField( CONFIGVALUE_LOCPROVIDER );
	  if( selectedProvider != null )
	     for( int i = 0 ; i < locProviderNames.length ; ++i )
			if( selectedProvider.equals( locProviderNames[i] ) ) {
			  selectedProviderIndex = i;
			  break;
			}
      ArrayAdapter<String> adapter = new ArrayAdapter<String>(this,
                android.R.layout.simple_spinner_item, locProviderNames );
      adapter.setDropDownViewResource(
		android.R.layout.simple_spinner_dropdown_item);
      locselector.setAdapter(adapter);
	  if( selectedProviderIndex >= 0 )
		locselector.setSelection( selectedProviderIndex );

// Setting up the sync server address edit field
	  String syncServerAddress = getField( CONFIGVALUE_SYNCADDRESS );
	  if( syncServerAddress == null )
		syncServerAddress = DEFAULTVALUE_SYNCADDRESS;
	  EditText syncServerAddressControl = (EditText)findViewById( R.id.sync_address );
	  syncServerAddressControl.setText( syncServerAddress );	  

// Setting up the sync user edit field
	  String syncUser = getField( CONFIGVALUE_SYNCUSER );
	  if( syncUser == null )
		syncUser = DEFAULTVALUE_SYNCUSER;
	  EditText syncUserControl = (EditText)findViewById( R.id.sync_user );
	  syncUserControl.setText( syncUser );	  

// Setting up the sync user edit field
	  String syncPassword = getField( CONFIGVALUE_SYNCPASSWORD );
	  if( syncPassword == null )
		syncUser = DEFAULTVALUE_SYNCPASSWORD;
	  EditText syncPasswordControl = (EditText)findViewById( R.id.sync_password );
	  syncPasswordControl.setText( syncPassword );	  

// Setting up warning timeout checkbox
	  String warningEnabledStr = getField( CONFIGVALUE_WARNINGENABLED );
	  if( warningEnabledStr == null )
		warningEnabledStr = DEFAULTVALUE_WARNINGENABLED;
	  CheckBox warningEnabledControl = (CheckBox)findViewById( R.id.warning_enabled );
	  warningEnabledControl.setChecked( warningEnabledStr.equals( "true" ) );
			
// Setting up the warning timeout edit field
	  String warningTimeoutStr = getField( CONFIGVALUE_WARNINGTIMEOUT );
	  if( warningTimeoutStr == null )
		warningTimeoutStr = Integer.toString( DEFAULTVALUE_WARNINGTIMEOUT );
	  EditText warningTimeoutControl = (EditText)findViewById( R.id.warning_timeout );
	  warningTimeoutControl.setText( warningTimeoutStr );

// Setting up the notification interval edit field
	  String notificationIntervalStr = getField( CONFIGVALUE_NOTIFICATIONINTERVAL );
	  if( notificationIntervalStr == null )
		notificationIntervalStr = Integer.toString( DEFAULTVALUE_NOTIFICATIONINTERVAL );
	  EditText notificationIntervalControl = (EditText)findViewById( R.id.notification_interval );
	  notificationIntervalControl.setText( notificationIntervalStr );

// Setting up the notification distance edit field
	  String notificationDistanceStr = getField( CONFIGVALUE_NOTIFICATIONDISTANCE );
	  if( notificationDistanceStr == null )
		notificationDistanceStr = Integer.toString( DEFAULTVALUE_NOTIFICATIONDISTANCE );
	  EditText notificationDistanceControl = (EditText)findViewById( R.id.notification_distance );
	  notificationDistanceControl.setText( notificationDistanceStr );

// Setting up the cancel button
      Button cancelButton = (Button)findViewById( R.id.cancel_button );
      cancelButton.setOnClickListener(new View.OnClickListener() {
         public void onClick(View view) {
			closeActivity( false );
         }
      } );
// Setting up the save button
      Button saveButton = (Button)findViewById( R.id.save_button );
      saveButton.setOnClickListener(new View.OnClickListener() {
         public void onClick(View view) {
			if( !saveConfig() )
				return;
			closeActivity( false );
         }
      } );
    }

	private String[] getLocationProviderNames() {
	  LocationManager lm = (LocationManager)getSystemService(Context.LOCATION_SERVICE);
	  List locProviderList = lm.getProviders();
	  String[] retValue = new String[ locProviderList.size() ];
	  for( int i = 0 ; i < locProviderList.size() ; ++i ) {
		LocationProvider locProvider = (LocationProvider)locProviderList.get( i );
		retValue[i] = locProvider.getName();
	  }
	  return retValue;
	}

	private void closeActivity( boolean save ) {
	  Bundle extras = new Bundle();
	  extras.putBoolean( "save",save );
      setResult( RESULT_OK, null, extras );
      finish();
	}

	private boolean saveConfig() {
// Save the spinner's selected entry
      Spinner locSelector = (Spinner)findViewById( R.id.locprovider_spinner );
	  int selectedItemId = locSelector.getSelectedItemPosition();
	  SpinnerAdapter sAdapter = locSelector.getAdapter();
	  String locProviderSelected = (String)sAdapter.getItem( selectedItemId );

	  EditText syncServerAddressControl = (EditText)findViewById( R.id.sync_address );
	  String  syncServerAddress = syncServerAddressControl.getText().toString();

	  EditText syncUserControl = (EditText)findViewById( R.id.sync_user );
	  String  syncUser = syncUserControl.getText().toString();

	  EditText syncPasswordControl = (EditText)findViewById( R.id.sync_password );
	  String syncPassword = syncPasswordControl.getText().toString();

	  CheckBox warningEnabledControl = (CheckBox)findViewById( R.id.warning_enabled );
	  String warningEnabled = 
		warningEnabledControl.isChecked() ? "true" : "false";
	  EditText warningTimeout = (EditText)findViewById( R.id.warning_timeout );
	  String s = warningTimeout.getText().toString();
	  int warningTimeoutInt = 0;
	  try {
		warningTimeoutInt = Integer.parseInt( s );
	  } catch( NumberFormatException ex ) {
		Toast.makeText(this, "Invalid warning timeout", Toast.LENGTH_SHORT).show();
		return false;
	  }
	  EditText notificationInterval = (EditText)findViewById( R.id.notification_interval );
	  s = notificationInterval.getText().toString();
	  int notificationIntervalInt = 0;
	  try {
		notificationIntervalInt = Integer.parseInt( s );
	  } catch( NumberFormatException ex ) {
		Toast.makeText(this, "Invalid tracking accuracy (sec)", Toast.LENGTH_SHORT).show();
		return false;
	  }
	  EditText notificationDistance = (EditText)findViewById( R.id.notification_distance );
	  s = notificationDistance.getText().toString();
	  int notificationDistanceInt = 0;
	  try {
		notificationDistanceInt = Integer.parseInt( s );
	  } catch( NumberFormatException ex ) {
		Toast.makeText(this, "Invalid tracking accuracy (meters)", Toast.LENGTH_SHORT).show();
		return false;
	  }
	  saveField( CONFIGVALUE_LOCPROVIDER,locProviderSelected,false );
	  saveField( CONFIGVALUE_SYNCADDRESS, syncServerAddress, false );
	  saveField( CONFIGVALUE_SYNCUSER, syncUser, false );
	  saveField( CONFIGVALUE_SYNCPASSWORD, syncPassword, false );
	  saveField( CONFIGVALUE_WARNINGENABLED, warningEnabled, false );
	  saveField( CONFIGVALUE_WARNINGTIMEOUT,
				Integer.toString( warningTimeoutInt ), false );
	  saveField( CONFIGVALUE_NOTIFICATIONINTERVAL,
				Integer.toString( notificationIntervalInt ), false );
	  saveField( CONFIGVALUE_NOTIFICATIONDISTANCE,
				Integer.toString( notificationDistanceInt ), true );
	  return true;
	}

	private void saveField( String fieldName, String value, boolean lastValue ) {
// Let's check whether the config value exists, update if it existed,
// insert, if it did not.
	  ContentResolver cr = getContentResolver();
	  String qColumns[] = { PeebleData.Settings._ID, 
							PeebleData.Settings.NAME,
							PeebleData.Settings.VALUE }; 
	  String whereArgs[] = new String[1];
	  whereArgs[0] = fieldName;
	  Cursor resultCursor = cr.query( 
			PeebleData.Settings.CONTENT_URI,
			qColumns,
			"name=?",
			whereArgs,
			null );
// if the value exists, we update it
	  if( ( resultCursor != null ) &&
		  ( resultCursor.next() ) ) {
		int valueColumnIndex =  
				resultCursor.getColumnIndex( PeebleData.Settings.VALUE );
		resultCursor.updateString( valueColumnIndex, value );
		resultCursor.commitUpdates();
		resultCursor.close();
	  } else {
// if the value does not exist, it is inserted
		ContentValues val = new ContentValues();
	    val.put( PeebleData.Settings.NAME, fieldName );
	    val.put( PeebleData.Settings.VALUE, value );
		if( lastValue )
			val.put( PeebleData.Settings.NOTIFY, "" );
		cr.insert( PeebleData.Settings.CONTENT_URI,val );
	  }
	}

	private String getField( String fieldName ) {
	  ContentResolver cr = getContentResolver();
	  String qColumns[] = { "name","value" }; 
	  String whereArgs[] = new String[1];
	  whereArgs[0] = fieldName;
	  Cursor resultCursor = cr.query( 
			PeebleData.Settings.CONTENT_URI,
			qColumns,
			"name=?",
			whereArgs,
			null );
// if the value exists, we update it
	  if( resultCursor == null )
			return null;
	  if( !resultCursor.next() )  {
		resultCursor.close();
		return null;
	  }
	  int valueColumnIndex =  resultCursor.getColumnIndex( "value" );
	  String valueValue = resultCursor.getString( valueColumnIndex );
	  resultCursor.close();
	  return valueValue;
	}
}
