package hu.uw.pallergabor.peeble;

import android.app.ListActivity;
import android.content.Intent;
import android.content.ContentResolver;
import android.content.ContentValues;
import android.content.ServiceConnection;
import android.content.Context;
import android.content.ComponentName;
import android.database.Cursor;
import android.net.Uri;
import android.os.Bundle;
import android.os.IBinder;
import android.os.Handler;
import android.os.DeadObjectException;
import android.util.Log;
import android.widget.SimpleAdapter;
import java.util.ArrayList;
import java.util.HashMap;

public class HighlightDisplay extends ListActivity {
	private static final String LOG_TAG = "Peeble_HighlightDisplay";
	private LocScanServiceConnection locScanServiceConnection = null;
	private ILocScanService locScanService = null;
	private String idLine = null;
	private Handler uiHandler = new Handler();

    /** Called when the activity is first created. */
    @Override
    public void onCreate(Bundle icicle) {
      super.onCreate(icicle);
	  idLine = (String)( getIntent().getExtra( "ids" ) );
	  bindService();
      setContentView( R.layout.highlight_display );
	  updateDisplay();
    }

	@Override
	public void onDestroy() {
	  super.onDestroy();
	  releaseService();
	}

	private void updateDisplay() {
	  ArrayList<HashMap<String,String>> itemList = new ArrayList<HashMap<String,String>>();
	  if( idLine != null ) {
		ContentResolver cr = getContentResolver();
		Log.d( LOG_TAG, "launch with argument: "+idLine );
		String ids[] = idLine.split( "," );
		for( int i = 0 ; i < ids.length ; ++i ) {
		  Uri queryUri = Uri.withAppendedPath( 
				PeebleData.LocHighlight.CONTENT_URI, ids[i] );
		  String columns[] = {
					PeebleData.LocHighlight.DESCRIPTION,
					PeebleData.LocHighlight.ADDRESS };
		  Cursor c = cr.query(
						queryUri,
						columns,
						null,
						null,
						null );
		  if( !c.next() )
			continue;
		  int descriptionColumnIndex = 
				c.getColumnIndex( PeebleData.LocHighlight.DESCRIPTION );
		  int addressColumnIndex =
				c.getColumnIndex( PeebleData.LocHighlight.ADDRESS );
		  String description = c.getString( descriptionColumnIndex );
		  String address = c.getString( addressColumnIndex );
		  HashMap<String,String> item = new HashMap<String,String>();
		  item.put( "line1",description );
		  item.put( "line2",address );
		  itemList.add( item );
		  c.close();
		}
// Now acknowledge the items
		for( int i = 0 ; i < ids.length ; ++i ) {
	  	  Uri updateUri = Uri.withAppendedPath( 
				PeebleData.LocHighlight.CONTENT_URI, 
				"displaytimestamp" );
	  	  updateUri = Uri.withAppendedPath( updateUri, ids[i] );
	  	  cr.update( updateUri, new ContentValues(), null, null );
		}
	  } else
		Log.d( LOG_TAG, "idLine==null" );
      SimpleAdapter listEntries = new SimpleAdapter( 
				this, 
				itemList,
				R.layout.main_item_two_line_row,
				new String[] { "line1","line2" },
				new int[] { R.id.text1, R.id.text2 }  );
      setListAdapter( listEntries );
    }

	private void bindService() {
	  if( locScanServiceConnection == null ) {
	    locScanServiceConnection = new LocScanServiceConnection();
	    Intent i = new Intent();
	    i.setClassName( "hu.uw.pallergabor.peeble", 
			"hu.uw.pallergabor.peeble.LocationScanningService" );
	    bindService( i, locScanServiceConnection, Context.BIND_AUTO_CREATE);
		Log.d( LOG_TAG, "bindService()" );
	  }
	}

	private void releaseService() {
	  if( locScanServiceConnection != null ) {
		if( locScanService != null ) {
		  try {
		    locScanService.removeHighlightCallback();
		  } catch( DeadObjectException ex ) {
			Log.e( LOG_TAG, "releaseService: DeadObjectException", ex );
		  }
		}
	    unbindService( locScanServiceConnection );	  
	    locScanServiceConnection = null;
	  }
	}

    class LocScanServiceConnection implements ServiceConnection {
        public void onServiceConnected(ComponentName className, 
			IBinder boundService ) {
          locScanService = ILocScanService.Stub.asInterface((IBinder)boundService);
		  Log.d( LOG_TAG,"LocationScanningService connected" );
		  try {
		    locScanService.setHighlightCallback( callbackService.asBinder() );
			idLine = locScanService.getLastHighlightIDs();
            uiHandler.post(new Runnable() {
				public void run() {
			 		updateDisplay();
                }
         	} );
		  } catch( DeadObjectException ex ) {
			Log.e( LOG_TAG, "onServiceConnected: DeadObjectException", ex );
		  }
        }

        public void onServiceDisconnected(ComponentName className) {
          locScanService = null;
		  Log.d( LOG_TAG,"LocationScanningService disconnected" );
        }
    };

    private IHighlightActivityCallback.Stub callbackService 
				= new IHighlightActivityCallback.Stub() {
	  public void remove() {
		finish();
	  }

	  public void updateIDs( String ids ) {
		 Log.d( LOG_TAG, "updateIDs: "+ids );
		 idLine = ids;
         uiHandler.post(new Runnable() {
           public void run() {
			 updateDisplay();
           }
         } );
	  }
    };

}
