package hu.uw.pallergabor.peeble;

import android.app.ListActivity;
import android.content.ContentResolver;
import android.content.Context;
import android.content.Intent;
import android.content.ServiceConnection;
import android.content.ComponentName;
import android.database.ContentObserver;
import android.database.Cursor;
import android.location.LocationProvider;
import android.location.LocationManager;
import android.net.Uri; 
import android.os.Bundle;
import android.os.Handler;
import android.os.IBinder;
import android.os.DeadObjectException;
import android.util.Log;
import android.view.Menu;
import android.view.Menu.Item;
import android.widget.SimpleAdapter;
import android.view.Menu;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

public class Peeble extends ListActivity {
    private static final String LOG_TAG = "Peeble";
	private static final int MENU_CONFIG_ACTIVITY = 1;
	private static final int MENU_START_LOC_SCANNING = 2;
	private static final int MENU_STOP_LOC_SCANNING = 3;
	private static final int MENU_SYNC = 4;
	private static final int MENU_TERMS_ACTIVITY = 5;

	public static final int ACTIVITY_CONFIG = 1;
	public static final int ACTIVITY_TERMS = 2;

	private LocDistroObserver locDistroObserver = null;
	private Handler uiHandler = null;
    private ILocScanService locScanService = null;
    private LocScanServiceConnection locScanServiceConnection = null;

	class LocDistroObserver extends ContentObserver {
	  public LocDistroObserver( Handler h ) {
		super( h );
	  }

	  public void onChange(boolean selfChange) {
		Log.d( LOG_TAG, "LogDistroObserver.onChange( "+selfChange+")" );
		if( locScanService != null )
			updateDisplay();
	  }
	}

    class LocScanServiceConnection implements ServiceConnection {
        public void onServiceConnected(ComponentName className, 
			IBinder boundService ) {
          locScanService = ILocScanService.Stub.asInterface((IBinder)boundService);
		  Log.d( LOG_TAG,"LocationScanningService connected" );
        }

        public void onServiceDisconnected(ComponentName className) {
          locScanService = null;
		  Log.d( LOG_TAG,"LocationScanningService disconnected" );
        }
    };


    /** Called when the activity is first created. */
    @Override
    public void onCreate(Bundle icicle) {
      super.onCreate(icicle);
	  ArrayList initList = new ArrayList();
      SimpleAdapter listEntries = new SimpleAdapter( 
				this, 
				initList,
				R.layout.main_item_two_line_row,
				new String[] { "line1","line2" },
				new int[] { R.id.text1, R.id.text2 }  );
      setListAdapter( listEntries );
      setContentView(R.layout.main);
    }

	@Override
	protected void onStart() {
	  super.onStart();
	  uiHandler = new Handler();
	  ContentResolver cr = getContentResolver();
	  locDistroObserver = new LocDistroObserver( uiHandler );
	  cr.registerContentObserver( PeebleData.LocDistro.CONTENT_URI, true,
							locDistroObserver );
	  bindService();
	}

	@Override
	protected void onStop() {
	  super.onStop();
	  ContentResolver cr = getContentResolver();
	  if( locDistroObserver != null ) {		// just paranoia
		cr.unregisterContentObserver( locDistroObserver );
		locDistroObserver = null;
	  }
	  releaseService();
	}

    @Override
    public boolean onCreateOptionsMenu(Menu menu) {
        boolean result = super.onCreateOptionsMenu(menu);
		menu.add(0, MENU_CONFIG_ACTIVITY, R.string.config );
		menu.add(0, MENU_TERMS_ACTIVITY, R.string.terms );
		menu.add(0, MENU_SYNC, R.string.sync );
		menu.add(0, MENU_START_LOC_SCANNING, R.string.start_loc_scanning );
//		menu.add(0, MENU_STOP_LOC_SCANNING, R.string.stop_loc_scanning );
        return result;
    }

    @Override
    public boolean onOptionsItemSelected(Item item) {
		int id = item.getId();
    	switch ( id ) {
    	  case MENU_CONFIG_ACTIVITY:
    		startConfig();
    		break;

		  case MENU_START_LOC_SCANNING:
			startLocationScanning();
			break;

		  case MENU_STOP_LOC_SCANNING:
			stopLocationScanning();
			break;

		  case MENU_TERMS_ACTIVITY:
			startTerms();
			break;

		  case MENU_SYNC:
			startSync();
			break;
    	}
        return super.onOptionsItemSelected(item);
    }


    @Override
    protected void onActivityResult(int requestCode, int resultCode, 
				String data, Bundle extras) {
      super.onActivityResult(requestCode, resultCode, data, extras);
      switch(requestCode) {
        case ACTIVITY_CONFIG:
                break;
      }
    }

    private void startLocationScanning() {
// We always start the service whenever the location provider changes
// Only the first start results in the creation of the service as the service
// is a singleton
	  Intent i = new Intent();
	  i.setClassName( "hu.uw.pallergabor.peeble", 
			"hu.uw.pallergabor.peeble.LocationScanningService" );
	  Bundle b = new Bundle();
	  Log.d( LOG_TAG,"startService" );
	  startService( i,b );
	}

    private void stopLocationScanning() {
	  Intent i = new Intent();
	  i.setClassName( "hu.uw.pallergabor.peeble", 
			"hu.uw.pallergabor.peeble.LocationScanningService" );
	  Log.d( LOG_TAG,"stopService" );
	  stopService( i );
	}

    private void startConfig() {
	  Intent i = new Intent();
	  i.setClassName( "hu.uw.pallergabor.peeble", 
			"hu.uw.pallergabor.peeble.Config" );
      startSubActivity(i, ACTIVITY_CONFIG );
	}

    private void startTerms() {
	  Intent i = new Intent();
	  i.setClassName( "hu.uw.pallergabor.peeble", 
			"hu.uw.pallergabor.peeble.Terms" );
      startSubActivity(i, ACTIVITY_TERMS );
	}

	private void startSync() {
	  if( locScanService == null ) {
		Log.e( LOG_TAG, "locScanService==null" );
	    return;
	  }
	  try {
		locScanService.startSync();
	  } catch( DeadObjectException ex ) {
		Log.e( LOG_TAG, "startSync", ex );
	  }
	}

	private void bindService() {
	  if( locScanServiceConnection == null ) {
	    locScanServiceConnection = new LocScanServiceConnection();
	    Intent i = new Intent();
	    i.setClassName( "hu.uw.pallergabor.peeble", 
			"hu.uw.pallergabor.peeble.LocationScanningService" );
	    bindService( i, locScanServiceConnection, Context.BIND_AUTO_CREATE);
		Log.d( LOG_TAG, "bindService()" );
	  }
	}

	private void releaseService() {
	  if( locScanServiceConnection != null ) {
	    unbindService( locScanServiceConnection );	  
	    locScanServiceConnection = null;
	  }
	}

	private void updateDisplay() {
	  try {
	    Map coord = locScanService.getLastFix();
	    String latitude = (String)coord.get( "latitude" );
	    String longitude = (String)coord.get( "longitude" );
	    Log.d( LOG_TAG, "updateDisplay: longitude: "+longitude+
							",latitude: "+latitude );
		fillList( longitude,latitude );
	  } catch( DeadObjectException ex ) {
		Log.e( LOG_TAG, "DeadObjectException, updateDisplay()" );
	  }
	}

    private void fillList( String longitude, String latitude ) {
	  ArrayList<HashMap<String,String>> list = new ArrayList<HashMap<String,String>>();
	  String[] projection = new String[] {
    	PeebleData.LocHighlight.ADDRESS,
		PeebleData.LocHighlight.DESCRIPTION
	  };
	  Uri queryUri = Uri.withAppendedPath( PeebleData.LocHighlight.CONTENT_URI, 
					"location" );
	  queryUri = Uri.withAppendedPath( queryUri,longitude+","+latitude );
      Cursor managedCursor = managedQuery( 
//					PeebleData.LocHighlight.CONTENT_URI,
					queryUri,
                    projection, 
                    null,
                    PeebleData.LocHighlight.DEFAULT_SORT_ORDER );
      int addressColumn = 
		managedCursor.getColumnIndex( PeebleData.LocHighlight.ADDRESS ); 
      int descriptionColumn = 
		managedCursor.getColumnIndex( PeebleData.LocHighlight.DESCRIPTION ); 
	  while( managedCursor.next() ) {
		String address = managedCursor.getString( addressColumn );
		String description = managedCursor.getString( descriptionColumn );
		HashMap<String,String> item = new HashMap<String,String>();
		item.put( "line1", address );
		item.put( "line2", description );
		list.add( item );
//		Log.d( LOG_TAG, "longitude: "+longitude+
//				",latitude: "+latitude+" listLine: "+listLine );
	  }
	  managedCursor.close();
      SimpleAdapter listEntries = new SimpleAdapter( 
				this, 
				list,
				R.layout.main_item_two_line_row,
				new String[] { "line1","line2" },
				new int[] { R.id.text1, R.id.text2 }  );
	  setListAdapter( listEntries );
	}
}
