package hu.uw.pallergabor.peeble;

import android.net.Uri;
import android.provider.BaseColumns;
import android.provider.SyncColumns;

/**
 * Convenience definitions for LocationDistroProvider
 */
public final class PeebleData {
/**
  * Sort order for all the tables defined in this class
  */
    public static final String DEFAULT_SORT_ORDER = "_id DESC";


	public interface ExtendedSyncColumns extends SyncColumns {
	  public static final String _SYNC_DELETED = "_sync_deleted";
	  public static final String _SYNC_CREATED = "_sync_created";
	}

    /**
     * Location distribution table
     */

    public static final class LocDistro implements BaseColumns,ExtendedSyncColumns {
/**
  * Possible URI formats:
  * content://hu.uw.pallergabor.peeble.PeebleData/LocDistro
  * content://hu.uw.pallergabor.peeble.PeebleData/LocDistro/13
  * content://hu.uw.pallergabor.peeble.PeebleData/LocDistro/location/46.230,13.720
  */

        public static final Uri CONTENT_URI
                = Uri.parse("content://hu.uw.pallergabor.peeble.PeebleData/LocDistro");

        public static final Uri SYNCMETAINFO_CONTENT_URI
                = Uri.parse("content://hu.uw.pallergabor.peeble.PeebleData/LocDistro/syncmetainfo");

		// Center longitude for this row
        public static final String LONGITUDE_CENTER = "longitude";

		// Center latitude for this row
        public static final String LATITUDE_CENTER = "latitude";

		// Incidence counter for this row
        public static final String COUNTER = "counter";
    }


    /*
     * Location highlight table  
     */
	public static final class LocHighlight implements BaseColumns, ExtendedSyncColumns {
        public static final Uri CONTENT_URI
                = Uri.parse("content://hu.uw.pallergabor.peeble.PeebleData/LocHighlight");

        public static final Uri SYNCMETAINFO_CONTENT_URI
                = Uri.parse("content://hu.uw.pallergabor.peeble.PeebleData/LocHighlight/syncmetainfo");

		// Center longitude for this location
        public static final String LONGITUDE_BLOCK = "longitude_block";

		// Center latitude for this location
        public static final String LATITUDE_BLOCK = "latitude_block";

		// Center longitude for this location
        public static final String EXACT_LONGITUDE = "exact_longitude";

		// Center latitude for this location
        public static final String EXACT_LATITUDE = "exact_latitude";

		// Matching terms, separated by comma (,)
	    public static final String MATCHING_TERMS = "matching_terms";

		// Description of the highlighted location
		public static final String DESCRIPTION = "description";

		// Address of the highlighted location
		public static final String ADDRESS = "address";

		// Timestamp of the last display of the entry
		public static final String DISPLAY_TIMESTAMP = "display_timestamp";

		// Default sort order
		public static final String DEFAULT_SORT_ORDER = "_id DESC";
	}

	/**
      * Synchronization meta-info table. In our application, LocDistro and
      * LocHighlight tables have associated sync meta-info tables. 
	  * Sync meta-info tables store names and values.
      * Addressing:
	  * <baseuri>/syncmetainfo
      * <baseuri>/syncmetainfo/<id>
	  * where <baseuri> is the base URI of the table
      * (e.g. content://hu.uw.pallergabor.peeble.PeebleData/LocDistro)
      */ 
   public static final class SyncMetaInfo implements BaseColumns {
// Name of the meta-info element
		public static final String NAME = "name";
// Value of the meta-info element
		public static final String VALUE = "value";
   }


	/*
	 * Settings table
	 */
	public static final class Settings implements BaseColumns {
        public static final Uri CONTENT_URI
                = Uri.parse("content://hu.uw.pallergabor.peeble.PeebleData/Settings");
		// name of the property
        public static final String NAME = "name";
		// value of the property
		public static final String VALUE = "value";

		// notification flag, if present, content resolver will broadcast
		// content change notification callbacks
		public static final String NOTIFY = "notify";
	}

	/*
	 * Terms table
	 */
	public static final class Terms implements BaseColumns, ExtendedSyncColumns {
        public static final Uri CONTENT_URI
                = Uri.parse("content://hu.uw.pallergabor.peeble.PeebleData/Terms");
        public static final Uri SYNCMETAINFO_CONTENT_URI
                = Uri.parse("content://hu.uw.pallergabor.peeble.PeebleData/Terms/syncmetainfo");
		// name of the property
        public static final String TERMS = "terms";
	}

}
