package hu.uw.pallergabor.peeble;

import android.content.ContentProvider;
import android.database.sqlite.SQLiteOpenHelper;
import android.content.ContentValues;
import android.content.IContentProvider;
import android.database.sqlite.SQLiteQueryBuilder;
import android.content.MergeResult;
import android.content.Context;
import android.content.UriMatcher;
import android.content.ContentUris;

import android.database.Cursor;
import android.database.SQLException;
import android.database.sqlite.SQLiteDatabase;
import android.net.Uri; 
import android.provider.BaseColumns;
import android.text.TextUtils;
import android.util.Log;
import java.util.HashMap;

public class PeebleDataProvider extends ContentProvider {
  private static final String DBNAME = "peebledata.db";
  private static final int DBVERSION = 1;
  private static final String LOG_TAG = "PeebleDataProvider";
  private SQLiteDatabase db;

  private static final UriMatcher URL_MATCHER;

  private static HashMap<String, String> LOCATIONDISTRO_PROJECTION_MAP;
  private static HashMap<String, String> LOCATIONDISTRO_SYNCMETA_PROJECTION_MAP;
  private static HashMap<String, String> LOCATIONHIGHLIGHT_PROJECTION_MAP;
  private static HashMap<String, String> LOCATIONHIGHLIGHT_SYNCMETA_PROJECTION_MAP;
  private static HashMap<String, String> SETTINGS_PROJECTION_MAP;
  private static HashMap<String, String> TERMS_PROJECTION_MAP;
  private static HashMap<String, String> TERMS_SYNCMETA_PROJECTION_MAP;

  private static final int LOCATIONDISTRO = 1;
  private static final int LOCATIONDISTRO_ID = 2;
  private static final int LOCATIONDISTRO_DELETE = 3;
  private static final int LOCATIONDISTRO_DELETE_ID = 4;
  private static final int LOCATIONDISTRO_LOC = 5;
  private static final int LOCATIONDISTRO_SYNCMETA = 6;
  private static final int LOCATIONDISTRO_SYNCMETA_ID = 7;
  private static final int LOCATIONHIGHLIGHT = 8;
  private static final int LOCATIONHIGHLIGHT_ID = 9;
  private static final int LOCATIONHIGHLIGHT_DELETE = 10;
  private static final int LOCATIONHIGHLIGHT_DELETE_ID = 11;
  private static final int LOCATIONHIGHLIGHT_LOC = 12;
  private static final int LOCATIONHIGHLIGHT_DTSTAMP = 13;
  private static final int LOCATIONHIGHLIGHT_SYNCMETA = 14;
  private static final int LOCATIONHIGHLIGHT_SYNCMETA_ID = 15;
  private static final int SETTINGS = 16;
  private static final int SETTINGS_ID = 17;
  private static final int TERMS = 18;
  private static final int TERMS_ID = 19;
  private static final int TERMS_DELETE = 20;
  private static final int TERMS_DELETE_ID = 21;
  private static final int TERMS_SYNCMETA = 22;
  private static final int TERMS_SYNCMETA_ID = 23;

  static {
    URL_MATCHER = new UriMatcher(UriMatcher.NO_MATCH);

    URL_MATCHER.addURI("hu.uw.pallergabor.peeble.PeebleData", 
						"LocDistro", 
						LOCATIONDISTRO );
    URL_MATCHER.addURI("hu.uw.pallergabor.peeble.PeebleData", 
						"LocDistro/#", 
						LOCATIONDISTRO_ID );
    URL_MATCHER.addURI("hu.uw.pallergabor.peeble.PeebleData", 
						"LocDistro/delete", 
						LOCATIONDISTRO_DELETE );
    URL_MATCHER.addURI("hu.uw.pallergabor.peeble.PeebleData", 
						"LocDistro/delete/#", 
						LOCATIONDISTRO_DELETE_ID );
    URL_MATCHER.addURI("hu.uw.pallergabor.peeble.PeebleData", 
						"LocDistro/location/*", 
						LOCATIONDISTRO_LOC );
    URL_MATCHER.addURI("hu.uw.pallergabor.peeble.PeebleData", 
						"LocDistro/syncmetainfo", 
						LOCATIONDISTRO_SYNCMETA );
    URL_MATCHER.addURI("hu.uw.pallergabor.peeble.PeebleData", 
						"LocDistro/syncmetainfo/#", 
						LOCATIONDISTRO_SYNCMETA_ID );
	URL_MATCHER.addURI("hu.uw.pallergabor.peeble.PeebleData",
						"LocHighlight",
						LOCATIONHIGHLIGHT );
	URL_MATCHER.addURI("hu.uw.pallergabor.peeble.PeebleData",
						"LocHighlight/#",
						LOCATIONHIGHLIGHT_ID );
    URL_MATCHER.addURI("hu.uw.pallergabor.peeble.PeebleData", 
						"LocHighlight/delete", 
						LOCATIONHIGHLIGHT_DELETE );
    URL_MATCHER.addURI("hu.uw.pallergabor.peeble.PeebleData", 
						"LocHighlight/delete/#", 
						LOCATIONHIGHLIGHT_DELETE_ID );
	URL_MATCHER.addURI("hu.uw.pallergabor.peeble.PeebleData",
						"LocHighlight/location/*",
						LOCATIONHIGHLIGHT_LOC );
	URL_MATCHER.addURI("hu.uw.pallergabor.peeble.PeebleData",
						"LocHighlight/displaytimestamp/#",
						LOCATIONHIGHLIGHT_DTSTAMP );
    URL_MATCHER.addURI("hu.uw.pallergabor.peeble.PeebleData", 
						"LocHighlight/syncmetainfo", 
						LOCATIONHIGHLIGHT_SYNCMETA );
    URL_MATCHER.addURI("hu.uw.pallergabor.peeble.PeebleData", 
						"LocHighlight/syncmetainfo/#", 
						LOCATIONHIGHLIGHT_SYNCMETA_ID );
	URL_MATCHER.addURI("hu.uw.pallergabor.peeble.PeebleData",
						"Settings",
						SETTINGS );
	URL_MATCHER.addURI("hu.uw.pallergabor.peeble.PeebleData",
						"Settings/#",
						SETTINGS_ID );
	URL_MATCHER.addURI("hu.uw.pallergabor.peeble.PeebleData",
						"Terms",
						TERMS );
	URL_MATCHER.addURI("hu.uw.pallergabor.peeble.PeebleData",
						"Terms/#",
						TERMS_ID );
	URL_MATCHER.addURI("hu.uw.pallergabor.peeble.PeebleData",
						"Terms/delete",
						TERMS_DELETE );
	URL_MATCHER.addURI("hu.uw.pallergabor.peeble.PeebleData",
						"Terms/delete/#",
						TERMS_DELETE_ID );
	URL_MATCHER.addURI("hu.uw.pallergabor.peeble.PeebleData",
						"Terms/syncmetainfo",
						TERMS_SYNCMETA );
	URL_MATCHER.addURI("hu.uw.pallergabor.peeble.PeebleData",
						"Terms/syncmetainfo/#",
						TERMS_SYNCMETA_ID );

    LOCATIONDISTRO_PROJECTION_MAP = new HashMap<String, String>();
    LOCATIONDISTRO_PROJECTION_MAP.put( 
		PeebleData.LocDistro._ID, "_id");
    LOCATIONDISTRO_PROJECTION_MAP.put( 
		PeebleData.LocDistro.LONGITUDE_CENTER, "longitude");
    LOCATIONDISTRO_PROJECTION_MAP.put( 
		PeebleData.LocDistro.LATITUDE_CENTER, "latitude");
    LOCATIONDISTRO_PROJECTION_MAP.put( 
		PeebleData.LocDistro.COUNTER, "counter");
	LOCATIONDISTRO_PROJECTION_MAP.put( 
		PeebleData.LocDistro._SYNC_VERSION,"_sync_version" );

	LOCATIONDISTRO_SYNCMETA_PROJECTION_MAP = new HashMap<String, String>();
    LOCATIONDISTRO_SYNCMETA_PROJECTION_MAP.put( 
		PeebleData.SyncMetaInfo._ID, "_id");
    LOCATIONDISTRO_SYNCMETA_PROJECTION_MAP.put( 
		PeebleData.SyncMetaInfo.NAME, "name" );
    LOCATIONDISTRO_SYNCMETA_PROJECTION_MAP.put( 
		PeebleData.SyncMetaInfo.VALUE, "value" );

	LOCATIONHIGHLIGHT_PROJECTION_MAP = new HashMap<String, String>();
    LOCATIONHIGHLIGHT_PROJECTION_MAP.put( 
		PeebleData.LocHighlight._ID, "_id");
    LOCATIONHIGHLIGHT_PROJECTION_MAP.put( 
		PeebleData.LocHighlight.LONGITUDE_BLOCK, "longitude_block");
    LOCATIONHIGHLIGHT_PROJECTION_MAP.put( 
		PeebleData.LocHighlight.LATITUDE_BLOCK, "latitude_block");
    LOCATIONHIGHLIGHT_PROJECTION_MAP.put( 
		PeebleData.LocHighlight.EXACT_LONGITUDE, "exact_longitude");
    LOCATIONHIGHLIGHT_PROJECTION_MAP.put( 
		PeebleData.LocHighlight.EXACT_LATITUDE, "exact_latitude");
    LOCATIONHIGHLIGHT_PROJECTION_MAP.put( 
		PeebleData.LocHighlight.MATCHING_TERMS, "matching_terms");
    LOCATIONHIGHLIGHT_PROJECTION_MAP.put( 
		PeebleData.LocHighlight.DESCRIPTION, "description");
    LOCATIONHIGHLIGHT_PROJECTION_MAP.put( 
		PeebleData.LocHighlight.ADDRESS, "address");
    LOCATIONHIGHLIGHT_PROJECTION_MAP.put( 
		PeebleData.LocHighlight.DISPLAY_TIMESTAMP, "display_timestamp");
	LOCATIONHIGHLIGHT_PROJECTION_MAP.put( 
		PeebleData.LocHighlight._SYNC_VERSION,"_sync_version" );

	LOCATIONHIGHLIGHT_SYNCMETA_PROJECTION_MAP = new HashMap<String, String>();
    LOCATIONHIGHLIGHT_SYNCMETA_PROJECTION_MAP.put( 
		PeebleData.SyncMetaInfo._ID, "_id");
    LOCATIONHIGHLIGHT_SYNCMETA_PROJECTION_MAP.put( 
		PeebleData.SyncMetaInfo.NAME, "name" );
    LOCATIONHIGHLIGHT_SYNCMETA_PROJECTION_MAP.put( 
		PeebleData.SyncMetaInfo.VALUE, "value" );


	SETTINGS_PROJECTION_MAP = new HashMap<String, String>();
    SETTINGS_PROJECTION_MAP.put( 
		PeebleData.Settings._ID, "_id");
    SETTINGS_PROJECTION_MAP.put( 
		PeebleData.Settings.NAME, "name" );
    SETTINGS_PROJECTION_MAP.put( 
		PeebleData.Settings.VALUE, "value" );

	TERMS_PROJECTION_MAP = new HashMap<String, String>();
    TERMS_PROJECTION_MAP.put( 
		PeebleData.Terms._ID, "_id");
    TERMS_PROJECTION_MAP.put( 
		PeebleData.Terms.TERMS, "terms" );
	TERMS_PROJECTION_MAP.put( 
		PeebleData.Terms._SYNC_VERSION,"_sync_version" );

	TERMS_SYNCMETA_PROJECTION_MAP = new HashMap<String, String>();
    TERMS_SYNCMETA_PROJECTION_MAP.put( 
		PeebleData.SyncMetaInfo._ID, "_id");
    TERMS_SYNCMETA_PROJECTION_MAP.put( 
		PeebleData.SyncMetaInfo.NAME, "name" );
    TERMS_SYNCMETA_PROJECTION_MAP.put( 
		PeebleData.SyncMetaInfo.VALUE, "value" );
  }

   private static class PeebleDataDatabaseHelper 
					extends SQLiteOpenHelper {
	@Override
    public void onCreate( SQLiteDatabase db ) {
      db.execSQL("CREATE TABLE locdistro ( _id INTEGER PRIMARY KEY,"
                    + "longitude TEXT, latitude TEXT, counter TEXT,"
					+" _sync_version INTEGER,"
					+" _sync_deleted INTEGER,"
					+" _sync_created INTEGER);");
	  db.execSQL("CREATE TABLE locdistro_syncmeta ( _id INTEGER PRIMARY KEY,"
					+" name TEXT, value TEXT);" );
	  db.execSQL("CREATE TABLE lochighlight ( _id INTEGER PRIMARY KEY,"
					+" longitude_block TEXT, latitude_block TEXT,"
					+" exact_longitude TEXT, exact_latitude TEXT,"
					+" matching_terms TEXT, description TEXT,"
					+" address TEXT, "
					+" display_timestamp INTEGER, "
					+" _sync_version INTEGER,"
					+" _sync_deleted INTEGER,"
					+" _sync_created INTEGER );" );
	  db.execSQL("CREATE TABLE lochighlight_syncmeta ( _id INTEGER PRIMARY KEY,"
					+" name TEXT, value TEXT);" );
	  db.execSQL("CREATE TABLE settings ( _id INTEGER PRIMARY KEY,"
					+" name TEXT, value TEXT);" );
	  db.execSQL("CREATE TABLE terms ( _id INTEGER PRIMARY KEY,"
					+" terms TEXT,"
					+" _sync_version INTEGER,"
					+" _sync_deleted INTEGER,"
					+" _sync_created INTEGER);" );
	  db.execSQL("CREATE TABLE terms_syncmeta ( _id INTEGER PRIMARY KEY,"
					+" name TEXT, value TEXT);" );
	}

	@Override
    public void onUpgrade(SQLiteDatabase db, 
			int oldVersion, int newVersion) {
      Log.w( LOG_TAG, "Upgrading database from version " + 
					oldVersion + " to "
                    + newVersion + 
					", destroying all old data" );
      db.execSQL("DROP TABLE IF EXISTS locdistro");
	  db.execSQL("DROP TABLE IF EXISTS locdistro_syncmeta" );
      db.execSQL("DROP TABLE IF EXISTS lochighlight");
	  db.execSQL("DROP TABLE IF EXISTS lochighlight_syncmeta" );
      db.execSQL("DROP TABLE IF EXISTS settings");
      db.execSQL("DROP TABLE IF EXISTS terms");
	  db.execSQL("DROP TABLE IF EXISTS terms_syncmeta" );
      onCreate(db);
    }
  }

  @Override
  public boolean onCreate() {
        PeebleDataDatabaseHelper dbHelper = new PeebleDataDatabaseHelper();
        db = dbHelper.openDatabase(getContext(), 
			DBNAME, null, 
			DBVERSION);
        return ( db == null) ? false : true;
  }
  
	public Cursor query(Uri url,
			String[] projection, 
			String selection,
            String[] selectionArgs, 
			String sort) {
        SQLiteQueryBuilder qb = new SQLiteQueryBuilder();

        switch ( URL_MATCHER.match(url) ) {
          case LOCATIONDISTRO:
            qb.setTables( "locdistro" );
            qb.setProjectionMap(LOCATIONDISTRO_PROJECTION_MAP);
// Does not return tombstones
			qb.appendWhere( "_sync_deleted=0" );
            break;

          case LOCATIONDISTRO_ID:
            qb.setTables( "locdistro" );
            qb.appendWhere("_id=" + url.getPathSegments().get(1));
			qb.appendWhere( " AND _sync_deleted=0" );
            break;

          case LOCATIONDISTRO_DELETE:
            qb.setTables( "locdistro" );
            qb.setProjectionMap(LOCATIONDISTRO_PROJECTION_MAP);
            break;

          case LOCATIONDISTRO_DELETE_ID:
            qb.setTables( "locdistro" );
            qb.appendWhere("_id=" + url.getPathSegments().get(2));
            break;

		  case LOCATIONDISTRO_LOC: {
// Extracts longitude and latitude from the request URI
			  String loc = url.getPathSegments().get(2);
			  String parms[] = loc.split( "," );
			  if( parms.length < 2 )
				throw new IllegalArgumentException( 
					"Incorrect long/lat: "+loc+" in URI: "+url.toString() );
			  String sLong = getLongitudeCenter( parms[0] );
			  String sLat = getLatitudeCenter( parms[1] );
			  qb.setTables( "locdistro" );
			  qb.appendWhere( "longitude="+sLong+" AND latitude="+sLat );
			  qb.appendWhere( " AND _sync_deleted=0" );
			}
			break;

          case LOCATIONDISTRO_SYNCMETA:
            qb.setTables( "locdistro_syncmeta" );
            qb.setProjectionMap( LOCATIONDISTRO_SYNCMETA_PROJECTION_MAP );
            break;

          case LOCATIONDISTRO_SYNCMETA_ID:
            qb.setTables( "locdistro_syncmeta" );
            qb.appendWhere("_id=" + url.getPathSegments().get(2));
            break;

		  case LOCATIONHIGHLIGHT:
            qb.setTables( "lochighlight" );
            qb.setProjectionMap(LOCATIONHIGHLIGHT_PROJECTION_MAP);
			qb.appendWhere( "_sync_deleted=0" );
            break;

		  case LOCATIONHIGHLIGHT_ID:
            qb.setTables( "lochighlight" );
            qb.appendWhere("_id=" + url.getPathSegments().get(1));
			qb.appendWhere( " AND _sync_deleted=0" );
            break;

		  case LOCATIONHIGHLIGHT_DELETE:
            qb.setTables( "lochighlight" );
            qb.setProjectionMap(LOCATIONHIGHLIGHT_PROJECTION_MAP);
            break;

		  case LOCATIONHIGHLIGHT_DELETE_ID:
            qb.setTables( "lochighlight" );
            qb.appendWhere("_id=" + url.getPathSegments().get(2));
            break;


		  case LOCATIONHIGHLIGHT_LOC: {
// Extracts longitude and latitude from the request URI
			  String loc = url.getPathSegments().get(2);
			  String parms[] = loc.split( "," );
			  if( parms.length < 2 )
				throw new IllegalArgumentException( 
					"Incorrect long/lat: "+loc+" in URI: "+url.toString() );
			  String sLong = getLongitudeCenter( parms[0] );
			  String sLat = getLatitudeCenter( parms[1] );
			  qb.setTables( "lochighlight" );
			  qb.appendWhere( "longitude_block="+sLong+" AND latitude_block="+sLat );			
			  qb.appendWhere( " AND _sync_deleted=0" );
			}
			break;

          case LOCATIONHIGHLIGHT_SYNCMETA:
            qb.setTables( "lochighlight_syncmeta" );
            qb.setProjectionMap( LOCATIONHIGHLIGHT_SYNCMETA_PROJECTION_MAP );
            break;

          case LOCATIONHIGHLIGHT_SYNCMETA_ID:
            qb.setTables( "lochighlight_syncmeta" );
            qb.appendWhere("_id=" + url.getPathSegments().get(2));
            break;

          case SETTINGS:
            qb.setTables( "settings" );
            qb.setProjectionMap(SETTINGS_PROJECTION_MAP);
            break;

          case SETTINGS_ID:
            qb.setTables( "settings" );
            qb.appendWhere("_id=" + url.getPathSegments().get(1));
            break;

          case TERMS:
            qb.setTables( "terms" );
            qb.setProjectionMap(TERMS_PROJECTION_MAP);
// Does not return tombstones
			qb.appendWhere( "_sync_deleted=0" );
            break;

          case TERMS_ID:
            qb.setTables( "terms" );
            qb.appendWhere("_id=" + url.getPathSegments().get(1));
			qb.appendWhere( " AND _sync_deleted=0" );
            break;

          case TERMS_DELETE:
            qb.setTables( "terms" );
            qb.setProjectionMap(TERMS_PROJECTION_MAP);
            break;

          case TERMS_DELETE_ID:
            qb.setTables( "terms" );
            qb.appendWhere("_id=" + url.getPathSegments().get(2));
            break;

          case TERMS_SYNCMETA:
            qb.setTables( "terms_syncmeta" );
            qb.setProjectionMap( TERMS_SYNCMETA_PROJECTION_MAP );
            break;

          case TERMS_SYNCMETA_ID:
            qb.setTables( "terms_syncmeta" );
            qb.appendWhere("_id=" + url.getPathSegments().get(2));
            break;

          default:
            throw new IllegalArgumentException("Unknown URL " + url);
        }

        // If no sort order is specified use the default
        String orderBy;
        if (TextUtils.isEmpty(sort)) {
            orderBy = PeebleData.DEFAULT_SORT_ORDER;
        } else {
            orderBy = sort;
        }

        Cursor c = qb.query( db , projection, selection, selectionArgs, 
                null, null, orderBy );
        c.setNotificationUri(getContext().getContentResolver(), url);
        return c;
  }


  @Override
  public String getType(Uri url) {
    switch ( URL_MATCHER.match(url) ) {
      case LOCATIONDISTRO:
		return "vnd.android.cursor.dir/vnd.hu.uw.pallergabor.peeble.locdistro";

      case LOCATIONDISTRO_ID:
	  case LOCATIONDISTRO_LOC:
		return "vnd.android.cursor.item/vnd.hu.uw.pallergabor.peeble.locdistro";

	  case LOCATIONDISTRO_SYNCMETA:
	  case LOCATIONHIGHLIGHT_SYNCMETA:
		return "vnd.android.cursor.dir/vnd.hu.uw.pallergabor.peeble.syncmeta";

	  case LOCATIONDISTRO_SYNCMETA_ID:
	  case LOCATIONHIGHLIGHT_SYNCMETA_ID:
		return "vnd.android.cursor.item/vnd.hu.uw.pallergabor.peeble.syncmeta";

	  case LOCATIONHIGHLIGHT:
		return "vnd.android.cursor.dir/vnd.hu.uw.pallergabor.peeble.lochighlight";
	  	  
	  case LOCATIONHIGHLIGHT_ID:
	  case LOCATIONHIGHLIGHT_LOC:
		return "vnd.android.cursor.item/vnd.hu.uw.pallergabor.peeble.lochighlight";

	  case SETTINGS:
		return "vnd.android.cursor.dir/vnd.hu.uw.pallergabor.peeble.settings";

	  case SETTINGS_ID:
		return "vnd.android.cursor.item/vnd.hu.uw.pallergabor.peeble.settings";

	  case TERMS:
		return "vnd.android.cursor.dir/vnd.hu.uw.pallergabor.peeble.terms";

	  case TERMS_ID:
		return "vnd.android.cursor.item/vnd.hu.uw.pallergabor.peeble.terms";

      default:
            throw new IllegalArgumentException("Unknown URL " + url);
    }
  }

  @Override
  public Uri insert(Uri url, ContentValues initialValues) {
      long rowID;
      ContentValues values;

      if (initialValues != null)
            values = new ContentValues(initialValues);
      else
            values = new ContentValues();

// We accept only URIs in content://hu.uw.pallergabor.peeble.PeebleData/LocDistro
// or content:// ... /LocHighlight form

	  switch( URL_MATCHER.match(url) ) { 
		case LOCATIONDISTRO: {
// Check input parameters
			  if( !values.containsKey( PeebleData.LocDistro.LONGITUDE_CENTER ) )
				throw new IllegalArgumentException( "'longitude' parameter not present" );
			  if( !values.containsKey( PeebleData.LocDistro.LATITUDE_CENTER ) )
				throw new IllegalArgumentException( "'latitude' parameter not present" );
			  if( !values.containsKey( PeebleData.LocDistro.COUNTER ) )
				values.put( PeebleData.LocDistro.COUNTER, "1.0" );
			  addTimeStamp( values );
			  addCreatedDeletedTimeStamp( values );

// Normalize longitude and latitude into buckets
			  String longitudeCenter = 
				(String)values.get( PeebleData.LocDistro.LONGITUDE_CENTER );
	  		  longitudeCenter = getLongitudeCenter( longitudeCenter );
	  		  values.put( PeebleData.LocDistro.LONGITUDE_CENTER, longitudeCenter );
	  		  String latitudeCenter = 
				(String)values.get( PeebleData.LocDistro.LATITUDE_CENTER );
	  		  latitudeCenter = getLongitudeCenter( latitudeCenter );
	  		  values.put( PeebleData.LocDistro.LATITUDE_CENTER, latitudeCenter );

// Get counter
	  		  double counterValue = values.getAsDouble( PeebleData.LocDistro.COUNTER );

// Now check whether we already have a row for that bucket. If we have, update
// that row with 
			  double existingCounter = checkExistingCoordinates( 
					longitudeCenter, latitudeCenter,counterValue );
			  Log.d( LOG_TAG, "counterValue: "+counterValue+
					" ,existingCounter: "+existingCounter );
			  if( existingCounter >= 0.0 ) {
			    Uri uri = generateURI( false,longitudeCenter, latitudeCenter );
			    getContext().getContentResolver().notifyChange(uri, null);
			    return uri;
	  		  }
			  rowID = db.insert( "locdistro", "counter", values );
			  if( rowID > 0 ) {
			    Uri uri = generateURI( false,longitudeCenter, latitudeCenter );
			    getContext().getContentResolver().notifyChange(uri, null);
        	    return uri;
      	  	  }
			  throw new SQLException("Failed to insert row into " + url);
			}

		case LOCATIONHIGHLIGHT:
// Check input parameters
			if( !values.containsKey( PeebleData.LocHighlight.EXACT_LONGITUDE ) )
				throw new IllegalArgumentException( "'exact_longitude' parameter not present" );
			if( !values.containsKey( PeebleData.LocHighlight.EXACT_LATITUDE ) )
				throw new IllegalArgumentException( "'exact_latitude' parameter not present" );
			if( !values.containsKey( PeebleData.LocHighlight.MATCHING_TERMS ) )
				throw new IllegalArgumentException( "'matching_terms' parameter not present" );
			if( !values.containsKey( PeebleData.LocHighlight.DESCRIPTION ) )
				throw new IllegalArgumentException( "'description' parameter not present" );
			if( !values.containsKey( PeebleData.LocHighlight.ADDRESS ) )
				throw new IllegalArgumentException( "'address' parameter not present" );
			String longitudeCenter = 
				getLongitudeCenter( 
					(String)values.get( PeebleData.LocHighlight.EXACT_LONGITUDE ) );
			String latitudeCenter =
				getLatitudeCenter( 
					(String)values.get( PeebleData.LocHighlight.EXACT_LATITUDE ) );
			values.put( PeebleData.LocHighlight.LONGITUDE_BLOCK, longitudeCenter );
			values.put( PeebleData.LocHighlight.LATITUDE_BLOCK, latitudeCenter );
			addTimeStamp( values );
			addCreatedDeletedTimeStamp( values );
			rowID = db.insert( "lochighlight", "address", values );
			if( rowID > 0 ) {
			  Uri uri = generateURI( true, longitudeCenter, latitudeCenter );
			  getContext().getContentResolver().notifyChange(uri, null);
        	  return uri;
      	  	}
			throw new SQLException("Failed to insert row into " + url);
			

		case LOCATIONDISTRO_SYNCMETA: {
              if (values.containsKey( PeebleData.SyncMetaInfo.NAME ) == false)
                         values.put( PeebleData.SyncMetaInfo.NAME, "");
              if (values.containsKey( PeebleData.SyncMetaInfo.VALUE ) == false)
                         values.put( PeebleData.SyncMetaInfo.VALUE, "");
              rowID = db.insert( "locdistro_syncmeta", "name", values );
              if( rowID <= 0 )
                        throw new SQLException("Failed to insert row into " + url);
              Uri uri = Uri.withAppendedPath( 
					PeebleData.LocDistro.SYNCMETAINFO_CONTENT_URI,
					Long.toString( rowID ) );
        	  getContext().getContentResolver().notifyChange(uri, null);
	          return uri;  
			}

		case LOCATIONHIGHLIGHT_SYNCMETA: {
              if (values.containsKey( PeebleData.SyncMetaInfo.NAME ) == false)
                         values.put( PeebleData.SyncMetaInfo.NAME, "");
              if (values.containsKey( PeebleData.SyncMetaInfo.VALUE ) == false)
                         values.put( PeebleData.SyncMetaInfo.VALUE, "");
              rowID = db.insert( "lochighlight_syncmeta", "name", values );
              if( rowID <= 0 )
                        throw new SQLException("Failed to insert row into " + url);
              Uri uri = Uri.withAppendedPath( 
					PeebleData.LocHighlight.SYNCMETAINFO_CONTENT_URI,
					Long.toString( rowID ) );
        	  getContext().getContentResolver().notifyChange(uri, null);
	          return uri;  
			}


		case SETTINGS: {
              if (values.containsKey( PeebleData.Settings.NAME ) == false)
                         values.put( PeebleData.Settings.NAME, "");
              if (values.containsKey( PeebleData.Settings.VALUE ) == false)
                         values.put( PeebleData.Settings.VALUE, "");
// We notify content observers only if this key is present
			  boolean notify = false;
			  if( values.containsKey( PeebleData.Settings.NOTIFY ) ) {
				values.remove( PeebleData.Settings.NOTIFY );
				notify = true;
			  }

              rowID = db.insert( "settings", "name", values );
              if( rowID <= 0 )
                        throw new SQLException("Failed to insert row into " + url);
              Uri uri = Uri.withAppendedPath( 
					PeebleData.Settings.CONTENT_URI,
					Long.toString( rowID ) );
			  if( notify )
        	  	getContext().getContentResolver().notifyChange(uri, null);
	          return uri;
		    }

		case TERMS: {
              if (values.containsKey( PeebleData.Terms.TERMS ) == false)
                         values.put( PeebleData.Terms.TERMS, "");
			  addTimeStamp( values );
			  addCreatedDeletedTimeStamp( values );
              rowID = db.insert( "terms", "name", values );
              if( rowID <= 0 )
                        throw new SQLException("Failed to insert row into " + url);
              Uri uri = Uri.withAppendedPath( 
					PeebleData.Terms.CONTENT_URI,
					Long.toString( rowID ) );
        	  getContext().getContentResolver().notifyChange(uri, null);
	          return uri;
			}

		case TERMS_SYNCMETA: {
              if (values.containsKey( PeebleData.SyncMetaInfo.NAME ) == false)
                         values.put( PeebleData.SyncMetaInfo.NAME, "");
              if (values.containsKey( PeebleData.SyncMetaInfo.VALUE ) == false)
                         values.put( PeebleData.SyncMetaInfo.VALUE, "");
              rowID = db.insert( "terms_syncmeta", "name", values );
              if( rowID <= 0 )
                        throw new SQLException("Failed to insert row into " + url);
              Uri uri = Uri.withAppendedPath( 
					PeebleData.Terms.SYNCMETAINFO_CONTENT_URI,
					Long.toString( rowID ) );
        	  getContext().getContentResolver().notifyChange(uri, null);
	          return uri;  
			}

		default:
			throw new IllegalArgumentException("insert: Unknown URL " + url);

	  }
  }

  @Override
  public int delete(Uri url, String where, String[] whereArgs) {
    int count;
    long rowId = 0;
    switch (URL_MATCHER.match(url)) {
// In simple "delete" case in the locdistro, lochighlight and terms databases, 
// the entry is not really deleted but is turned into a "tombstone" by
// updating its _sync_deleted and _sync_version fields with the current
// timestamp
      case LOCATIONDISTRO: {
			  ContentValues val = tombstoneTimeStamp();
              count = db.update( "locdistro", val, where, whereArgs );
			}
            break;

	  case LOCATIONDISTRO_ID: {
              String segment = url.getPathSegments().get(1);
              rowId = Long.parseLong(segment);
			  ContentValues val = tombstoneTimeStamp();
              count = db.update( "locdistro", val, "_id="
                            + segment
                            + (!TextUtils.isEmpty(where) ? " AND (" + where
                                    + ')' : ""), whereArgs);
			}
			break;

      case LOCATIONDISTRO_DELETE:
            count = db.delete( "locdistro", where, whereArgs );
            break;

      case LOCATIONDISTRO_DELETE_ID: {
            String segment = url.getPathSegments().get(2);
            rowId = Long.parseLong(segment);
            count = db.delete( "locdistro", "_id="
                            + segment
                            + (!TextUtils.isEmpty(where) ? " AND (" + where
                                    + ')' : ""), whereArgs);
			}
            break;

	  case LOCATIONDISTRO_SYNCMETA:
			count = db.delete( "locdistro_syncmeta", where, whereArgs );
			break;

	  case LOCATIONDISTRO_SYNCMETA_ID: {
			  String segment = url.getPathSegments().get(2);
              rowId = Long.parseLong(segment);
              count = db.delete( "locdistro_syncmeta", "_id="
                            + segment
                            + (!TextUtils.isEmpty(where) ? " AND (" + where
                                    + ')' : ""), whereArgs);
			}
			break;

// Again, we leave tombstones for the lochighlight table for normal delete
// operations
      case LOCATIONHIGHLIGHT: {
			  ContentValues val = tombstoneTimeStamp();
              count = db.update( "lochighlight", val, where, whereArgs );
			}
            break;

	  case LOCATIONHIGHLIGHT_ID: {
              String segment = url.getPathSegments().get(1);
              rowId = Long.parseLong(segment);
			  ContentValues val = tombstoneTimeStamp();
              count = db.update( "lochighlight", val, "_id="
                            + segment
                            + (!TextUtils.isEmpty(where) ? " AND (" + where
                                    + ')' : ""), whereArgs);
			}
			break;

	  case LOCATIONHIGHLIGHT_DELETE:
			count = db.delete( "lochighlight", where, whereArgs );
			break;

	  case LOCATIONHIGHLIGHT_DELETE_ID: {
              String segment = url.getPathSegments().get(2);
              rowId = Long.parseLong(segment);
              count = db.delete( "lochighlight", "_id="
                            + segment
                            + (!TextUtils.isEmpty(where) ? " AND (" + where
                                    + ')' : ""), whereArgs);
			}
            break;

	  case LOCATIONHIGHLIGHT_SYNCMETA:
			count = db.delete( "lochighlight_syncmeta", where, whereArgs );
			break;

	  case LOCATIONHIGHLIGHT_SYNCMETA_ID: {
			  String segment = url.getPathSegments().get(2);
              rowId = Long.parseLong(segment);
              count = db.delete( "lochighlight_syncmeta", "_id="
                            + segment
                            + (!TextUtils.isEmpty(where) ? " AND (" + where
                                    + ')' : ""), whereArgs);
			}
			break;
	
	  case SETTINGS:
			count = db.delete( "settings", where, whereArgs );
			break;

	  case SETTINGS_ID: {
			  String segment = url.getPathSegments().get(1);
              rowId = Long.parseLong(segment);
              count = db.delete( "settings", "_id="
                            + segment
                            + (!TextUtils.isEmpty(where) ? " AND (" + where
                                    + ')' : ""), whereArgs);
			}
			break;

      case TERMS: {
			  ContentValues val = tombstoneTimeStamp();
              count = db.update( "terms", val, where, whereArgs );
			}
            break;

	  case TERMS_ID: {
              String segment = url.getPathSegments().get(1);
              rowId = Long.parseLong(segment);
			  ContentValues val = tombstoneTimeStamp();
              count = db.update( "terms", val, "_id="
                            + segment
                            + (!TextUtils.isEmpty(where) ? " AND (" + where
                                    + ')' : ""), whereArgs);
			}
			break;

      case TERMS_DELETE:
            count = db.delete( "terms", where, whereArgs );
            break;

      case TERMS_DELETE_ID: {
            String segment = url.getPathSegments().get(2);
            rowId = Long.parseLong(segment);
            count = db.delete( "terms", "_id="
                            + segment
                            + (!TextUtils.isEmpty(where) ? " AND (" + where
                                    + ')' : ""), whereArgs);
			}
            break;

	  case TERMS_SYNCMETA:
			count = db.delete( "terms_syncmeta", where, whereArgs );
			break;

	  case TERMS_SYNCMETA_ID: {
			  String segment = url.getPathSegments().get(2);
              rowId = Long.parseLong(segment);
              count = db.delete( "terms_syncmeta", "_id="
                            + segment
                            + (!TextUtils.isEmpty(where) ? " AND (" + where
                                    + ')' : ""), whereArgs);
			}
			break;

      default:
            throw new IllegalArgumentException("Unknown URL " + url);
    }
    getContext().getContentResolver().notifyChange(url, null);
    return count;
  }

  @Override
  public int update(Uri url,
		ContentValues values, 
		String where, 
		String[] whereArgs) {
    int count;
	int urlmatch = URL_MATCHER.match(url);
	boolean notify = true;
    switch (URL_MATCHER.match(url)) {
        case LOCATIONDISTRO: {
			  addTimeStamp( values );
              count = db.update("locdistro", values, where, whereArgs);
			}
            break;

        case LOCATIONDISTRO_ID: {
			  addTimeStamp( values );
              String segment = url.getPathSegments().get(1);
              count = db.update( "locdistro", values, "_id="
                            + segment
                            + (!TextUtils.isEmpty(where) ? " AND (" + where
                                    + ')' : ""), whereArgs);
			}
            break;

		case LOCATIONDISTRO_LOC: {
			  addTimeStamp( values );
			  String loc = url.getPathSegments().get(2);
			  String parms[] = loc.split( "," );
			  if( parms.length < 2 )
				throw new IllegalArgumentException( 
					"Incorrect long/lat: "+loc+" in URI: "+url.toString() );
			  String sLong = getLongitudeCenter( parms[0] );
			  String sLat = getLatitudeCenter( parms[1] );
	  		  if( !values.containsKey( PeebleData.LocDistro.COUNTER ) )
				values.put( PeebleData.LocDistro.COUNTER, "1.0" );
			  double counterVal = values.getAsDouble( PeebleData.LocDistro.COUNTER );
			  checkExistingCoordinates(  sLong, sLat, counterVal );
			  count = 1;
			}
            break;

		case LOCATIONDISTRO_SYNCMETA: {
              count = db.update("locdistro_syncmeta", values, where, whereArgs);
			}
			break;

		case LOCATIONDISTRO_SYNCMETA_ID: {
              String segment = url.getPathSegments().get(2);
              count = db.update( "locdistro_syncmeta", values, "_id="
                            + segment
                            + (!TextUtils.isEmpty(where) ? " AND (" + where
                                    + ')' : ""), whereArgs);
			}
			break;

	    case LOCATIONHIGHLIGHT: {
			  addTimeStamp( values );
              count = db.update("lochighlight", values, where, whereArgs);
			}
            break;

		case LOCATIONHIGHLIGHT_ID: {
			  addTimeStamp( values );
              String segment = url.getPathSegments().get(1);
              count = db.update( "lochighlight", values, "_id="
                            + segment
                            + (!TextUtils.isEmpty(where) ? " AND (" + where
                                    + ')' : ""), whereArgs);
			}
            break;

		case LOCATIONHIGHLIGHT_LOC: {
			  addTimeStamp( values );
			  String loc = url.getPathSegments().get(2);
			  String parms[] = loc.split( "," );
			  if( parms.length < 2 )
				throw new IllegalArgumentException( 
					"Incorrect long/lat: "+loc+" in URI: "+url.toString() );
			  String sLong = getLongitudeCenter( parms[0] );
			  String sLat = getLatitudeCenter( parms[1] );
              count = db.update( "lochighlight", values, 
								"longitude_block="+sLong+
								" AND latitude_block="+sLat, null);
			}
			break;

		case LOCATIONHIGHLIGHT_DTSTAMP: {
			  addTimeStamp( values );
              String segment = url.getPathSegments().get(2);
			  ContentValues val = new ContentValues();
			  val.put( "display_timestamp",
					new Long( System.currentTimeMillis() ) );
              count = db.update( "lochighlight", val, 
							"_id="+segment, 
							null );
			}
			break;

		case LOCATIONHIGHLIGHT_SYNCMETA: {
              count = db.update("lochighlight_syncmeta", values, where, whereArgs);
			}
			break;

		case LOCATIONHIGHLIGHT_SYNCMETA_ID: {
              String segment = url.getPathSegments().get(2);
              count = db.update( "lochighlight_syncmeta", values, "_id="
                            + segment
                            + (!TextUtils.isEmpty(where) ? " AND (" + where
                                    + ')' : ""), whereArgs);
			}
			break;

		case SETTINGS: {
			  if( values.containsKey( PeebleData.Settings.NOTIFY ) )
					values.remove( PeebleData.Settings.NOTIFY );
			  else
				notify = false;
              count = db.update("settings", values, where, whereArgs);
            }
			break;

		case SETTINGS_ID: {
              String segment = url.getPathSegments().get(1);
			  if( values.containsKey( PeebleData.Settings.NOTIFY ) )
					values.remove( PeebleData.Settings.NOTIFY );
			  else
				notify = false;
              count = db.update( "settings", values, "_id="
                            + segment
                            + (!TextUtils.isEmpty(where) ? " AND (" + where
                                    + ')' : ""), whereArgs);
			}
			break;

        case TERMS: {
			  addTimeStamp( values );
              count = db.update("terms", values, where, whereArgs);
			}
            break;

        case TERMS_ID: {
			  addTimeStamp( values );
              String segment = url.getPathSegments().get(1);
              count = db.update( "terms", values, "_id="
                            + segment
                            + (!TextUtils.isEmpty(where) ? " AND (" + where
                                    + ')' : ""), whereArgs);
			}
            break;

		case TERMS_SYNCMETA: {
              count = db.update("terms_syncmeta", values, where, whereArgs);
			}
			break;

		case TERMS_SYNCMETA_ID: {
              String segment = url.getPathSegments().get(2);
              count = db.update( "terms_syncmeta", values, "_id="
                            + segment
                            + (!TextUtils.isEmpty(where) ? " AND (" + where
                                    + ')' : ""), whereArgs);
			}
			break;

        default:
            throw new IllegalArgumentException("Unknown URL: " + url);
    }
	if( notify )
    	getContext().getContentResolver().notifyChange(url, null);
	return count;
  }

  private String getLongitudeCenter( String longString ) {
	double longitude = 0.0;
	try {
	  longitude = Double.parseDouble( longString );
	} catch( NumberFormatException ex ) {
	  return null;
	}
	longitude = Math.round( longitude * 1000.0 ) / 1000.0;
	return Double.toString( longitude );
  }

  private String getLatitudeCenter( String latString ) {
	double latitude = 0.0;
	try {
	  latitude = Double.parseDouble( latString );
	} catch( NumberFormatException ex ) {
	  return null;
	}
	latitude = Math.round( latitude * 1000.0 ) / 1000.0;
	return Double.toString( latitude );
  }

// Returns the counter belonging to the longitude/latitude bucket if
// row for the bucket already exists. Returns -1 otherwise. If its delta
// parameter is not 0, the delta is added to the row value and the method
// will return the updated value
  private double checkExistingCoordinates( 
		String longitudeCenter, 
		String latitudeCenter, 
		double delta ) {
	String columns[] = { "_id", "counter" };
	String[] selectionArgs = new String[2];
	selectionArgs[0] = longitudeCenter;
	selectionArgs[1] = latitudeCenter;
	Cursor c = db.query( "locdistro", columns, "longitude=? AND latitude=?",
						selectionArgs, null, null, null );
	if( !c.next() ) {
	  c.close();
	  return -1;
	}
	int counterColIndex = c.getColumnIndex( "counter" );
	String colValueStr = c.getString( counterColIndex );
	double colValue = 0.0;
	try {
	  colValue = Double.parseDouble( colValueStr );
	} catch( NumberFormatException ex ) {
	  return -1;
	}
	if( delta != 0 ) {
	  colValue += delta;
	  c.updateString( counterColIndex,Double.toString( colValue ) );
	  c.commitUpdates();
	}
	c.close();
	return colValue;
  }

// Generates URI for a given longitude/latitude pair
  private Uri generateURI( boolean highlight, String longitudeCenter, String latitudeCenter ) {
	Uri baseURI = highlight ? 
					PeebleData.LocHighlight.CONTENT_URI :
					PeebleData.LocDistro.CONTENT_URI;
	Uri uri = Uri.withAppendedPath( baseURI, "location" );
	uri = Uri.withAppendedPath( uri,longitudeCenter+","+latitudeCenter );
	return uri;
  }

  private void addTimeStamp( ContentValues values ) {
	if( !values.containsKey( PeebleData.ExtendedSyncColumns._SYNC_VERSION ) )
		values.put( PeebleData.ExtendedSyncColumns._SYNC_VERSION,
			Long.toString( System.currentTimeMillis() ) );
  }

  private void addCreatedDeletedTimeStamp( ContentValues values ) {
	if( !values.containsKey( PeebleData.ExtendedSyncColumns._SYNC_DELETED ) )
		values.put( PeebleData.ExtendedSyncColumns._SYNC_DELETED,
				"0" );
	if( !values.containsKey( PeebleData.ExtendedSyncColumns._SYNC_CREATED ) )
		values.put( PeebleData.ExtendedSyncColumns._SYNC_CREATED,
			Long.toString( System.currentTimeMillis() ) );
  }

  private ContentValues tombstoneTimeStamp() {
	ContentValues val = new ContentValues();
	Long ts = new Long( System.currentTimeMillis() );
	val.put( PeebleData.ExtendedSyncColumns._SYNC_DELETED,ts );
	val.put( PeebleData.ExtendedSyncColumns._SYNC_CREATED,ts );
	return val;
  }
}
