package hu.uw.pallergabor.peeble;

import android.app.ListActivity;
import android.content.Intent;
import android.content.ContentResolver;
import android.content.ContentValues;
import android.content.Context;
import android.content.ComponentName;
import android.database.Cursor;
import android.net.Uri;
import android.os.Bundle;
import android.os.Handler;
import android.util.Log;
import android.view.Menu;
import android.view.Menu.Item;
import android.view.View;
import android.widget.ListAdapter;
import android.widget.ArrayAdapter;
import android.widget.Toast;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;

public class Terms extends ListActivity {
	private static final String LOG_TAG = "Peeble_Terms";
	private static final int ADD_TERM = 1;
	private static final int EDIT_TERM = 2;
	private static final int DELETE_TERM = 3;

	private static final int ACTIVITY_INVOKE = 1;

    /** Called when the activity is first created. */
    @Override
    public void onCreate(Bundle icicle) {
      super.onCreate(icicle);
      setContentView( R.layout.terms );
	  updateDisplay();
    }

	private void updateDisplay() {
	  ContentResolver cr = getContentResolver();
	  String columns[] = { PeebleData.Terms._ID, PeebleData.Terms.TERMS };
	  Cursor c = cr.query( 
					PeebleData.Terms.CONTENT_URI,
					columns,
					null,
					null,
					null );
      List<ListItem> items = new ArrayList<ListItem>();
	  while( c.next() ) {
		int idColumnIndex = c.getColumnIndex( PeebleData.Terms._ID );
        long id = c.getLong( idColumnIndex );
		int termsColumnIndex = c.getColumnIndex( PeebleData.Terms.TERMS );
        String terms = c.getString( termsColumnIndex );
		ListItem li = new ListItem( id, terms );
        items.add( li );
	  }
      ArrayAdapter<ListItem> termAdapter =
            new ArrayAdapter<ListItem>(this, R.layout.terms_row, items);
      setListAdapter( termAdapter );
    }

    @Override
    public boolean onCreateOptionsMenu(Menu menu) {
        boolean result = super.onCreateOptionsMenu(menu);
        menu.add(0, ADD_TERM, R.string.terms_add);
        menu.add(0, EDIT_TERM, R.string.terms_edit);
        menu.add(0, DELETE_TERM, R.string.terms_delete);
        return result;
    }

    @Override
    public boolean onOptionsItemSelected(Item item) {
        switch (item.getId()) {
          case ADD_TERM:
				addEditTermActivity( -1 );
                break;

		  case EDIT_TERM: {
				  long rowID = getCurrentRowID();
			      if( rowID >= 0 )
				  	addEditTermActivity( rowID );
				}
				break;

		  case DELETE_TERM: {
				  long rowID = getCurrentRowID();
				  if( rowID >= 0 )
				  	deleteTerm( rowID );
				}
				break;
        }

        return super.onOptionsItemSelected(item);
	}

    @Override
    protected void onActivityResult(int requestCode, int resultCode, 
							String data, Bundle extras) {
      super.onActivityResult(requestCode, resultCode, data, extras);

      switch(requestCode) {
            case ACTIVITY_INVOKE:
			   updateDisplay();
               break;
      }
    }

	private void addEditTermActivity( long id ) {
      Intent i = new Intent();
      i.setClassName( "hu.uw.pallergabor.peeble", "hu.uw.pallergabor.peeble.AddTerm" );
	  if( id >= 0 )
		i.putExtra( "id",new Long( id ) );
      startSubActivity(i, ACTIVITY_INVOKE);
	}

    private long getCurrentRowID() {
	  ListAdapter la = getListAdapter();
	  int idx = getSelectedItemPosition();
	  if( idx < 0 ) {
        Toast.makeText(this, "No item selected", Toast.LENGTH_SHORT).show();
		return -1;
	  }
	  else {
	    ListItem li = (ListItem)la.getItem( idx );
	    return li.id;
	  }
	}

	private void deleteTerm( long rowID ) {
	  ContentResolver cr = getContentResolver();
      Uri deleteUri = Uri.withAppendedPath( PeebleData.Terms.CONTENT_URI,
											Long.toString( rowID ) );
	  cr.delete( deleteUri, null, null );
	  updateDisplay();
	}

/**
  * Instances of this class are inserted into the list adapter array.
  * toString() yields the string field but it is also possible to retrieve
  * the id field which can be used to identify the item in the database.
  */
    class ListItem {
      public long id = 0;
	  public String item = null;

	  public ListItem( long id, String item ) {
		this.id = id;
		this.item = item;
	  }

      public String toString() {
		return item;
	  }
	}
}
