package mwfw.common;

import mwfw.ISendingProtocolAdapter;
import mwfw.IReceivingProtocolAdapter;
import mwfw.IReceivingDispatcher;
import mwfw.syncmlds.impl.Base64;
import java.io.OutputStream;
import java.io.InputStream;
import java.io.IOException;
import java.net.URL;
import java.net.URLConnection;
import java.util.HashMap;

class ConnMapEntry {
  public ConnMapEntry( URLConnection conn, String address ) {
    this.conn = conn;
    this.address = address;
  }

  public URLConnection conn;
  public String address;
}

/**
  * Component implementing HTTP-based sending protocol adapter.
  */
public class HttpProtocolAdapter implements 
					ISendingProtocolAdapter {
  ReceivingDispatcher rd = null;
  HashMap connmap = new HashMap();
  HashMap incomingconnmap = new HashMap();
  String listeneraddress = null;
  int handlectr = 0;

  public void init() {
  }

  public void shutdown() {
  }

  public void setReceivingDispatcher( ReceivingDispatcher rd ) {
    this.rd = rd;
  }

  public void unsetReceivingDispatcher( ReceivingDispatcher rd ) {
    this.rd = null;
  }

// ------------------ ISendingProtocolAdapter
  public OutputStream sendPackage( int handle, String address ) 
														throws Exception {
// This implementation may receive existing connections only if the connection was
// initiated by an incoming request. In this case the output stream must be present
// in incomingconnmap.
    if( handle != -1 ) {
      OutputStream os = (OutputStream)incomingconnmap.get( (Object)new Integer( handle ) );
      if( os == null )
      	throw new IOException( "os == null in HttpProtocolAdapter:sendPackage" );
      return os;
	}
	String authHeader = null;
	String addressPart = address;
	int qmidx = address.indexOf( '?' );
	if( qmidx >= 0 ) {
	  addressPart = address.substring( 0,qmidx );
	  if( qmidx+1 < address.length() ) {
		String parmPart = address.substring( qmidx+1 );
		String parms[] = parmPart.split( "&" );
		String username = null;
		String password = null;
		for( int i = 0 ; i < parms.length ; ++i ) {
		  String parm = parms[i];
		  int esidx = parm.indexOf( '=' );
		  if( ( esidx < 0 ) || ( esidx+1 >= parm.length() ) )
				continue;
		  String parmName = parm.substring( 0,esidx );
		  String parmValue = parm.substring( esidx+1 );
		  if( parmName.equals( "username" ) )
				username = parmValue;
		  else
		  if( parmName.equals( "password" ) )
				password = parmValue;
		}
		if( ( username != null ) && ( password != null ) ) {
		  String hdrval = username+":"+password;
		  authHeader = "Basic "+
						Base64.encodeBytes( hdrval.getBytes() );
		}
	  }
	}
    URL url = new URL( addressPart );
    URLConnection urlc = url.openConnection();
    urlc.setDoInput( true );
    urlc.setDoOutput( true );
    urlc.setUseCaches( true );
    urlc.setRequestProperty( "Content-Type","application/octet-stream" );
	if( authHeader != null )
		urlc.setRequestProperty( "Authorization",authHeader );
    OutputStream os = urlc.getOutputStream();
    ConnMapEntry cme = new ConnMapEntry( urlc, address );
    connmap.put( (Object)os,(Object)cme );
    return os;
  }

  public int closePackage( int handle, OutputStream pkg, IReceivingProtocolAdapter rpa ) throws Exception {
    pkg.flush();
 // In this case this was an incoming connection and the connection is closed in the service() method.
    if( handle != -1 ) {
      return -1;
    }
    URLConnection urlc = null;
    String address = null;
    synchronized( connmap ) {
      ConnMapEntry cme = (ConnMapEntry)connmap.get( (Object)pkg );
      urlc = cme.conn;
      address = cme.address;
      if( urlc == null )
		throw new IOException( "urlc == null in HttpProtocolAdapter:closePackage" ); 
      connmap.remove( (Object)pkg );
    }
    boolean handled = false;
    InputStream is = urlc.getInputStream();
    if( rpa != null )
      handled = rpa.receivePackage( -1, address, is );
    else
	if( rd != null )
		handled = rd.receivePackage( -1, address, is );
    if( !handled )
		is.close();
	return -1; 
  }
}
