package mwfw;

import java.io.InputStream;

/**
  * This interface is implemented by the receiving dispatcher and is actually
  * almost identical to the IReceivingProtocolAdapter interface. They are kept separate
  * to ensure that no malicious component can play the dispatcher role.
  * Protocol adapters send their incoming packages to the dispatcher that enforces
  * address filtering.
  */ 
public interface IReceivingDispatcher {
/**
  * Called by the protocol adapter if there is an incoming package.
  * @param handle Handle number if this package is associated with some existing connection
  *               or -1 if the bearer is connectionless or no previous connection exists.
  * @param address Package originator's address in URI format.
  * @param pkg The incoming package. This can be a real stream or a package of fixed
  *            length.
  * @return true if the package was handled.
  */
  public boolean receivePackage( int handle, String address, InputStream pkg ) throws Exception;
}
 