package mwfw;

import java.io.OutputStream;

/**
  * Interface for connectionless protocol adapters. This 
  * protocol adapter is able to send packages over connectionless or connection-oriented
  * bearer. In order to be able to find the appropriate protocol adapter, components
  * offering this interface must have mwfw.spa.id property and the value of this property must be a
  * unique URI identifying the adapter.
  */
public interface ISendingProtocolAdapter {
/**
  * Send a package to the communicating party. 
  * @param handle Handle number if the package to send is associated with some
  *        existing connection. This value must be -1 if the bearer is connectionless
  *        or new connection is always started.
  * @param address Address of the party to address in URI format.
  * @return OutputStream that can be used to write the package to send. 
  */
  public OutputStream sendPackage( int handle, String address ) throws Exception;

/**
  * Finishes the sending of the one package.
  * @param handle Handle of the package with which the output stream is associated with
  *               or -1 if the output stream is not associated to a connection.
  * @param pkg The OutputStream returned by sendPackage representing the package to
  *            close.
  * @param rpa If specified, the response package will be delivered only to the
  *        specified ReceivingProtocolAdapter. If this parameter is null,
  *        all the registered ReceivingProtocolAdapter will be offered the incoming package.
  * @return Handle if sending more packages using the connection is possible. This
  *         handle can be used as sendPackage parameter to refer to the connection.
  *         Returns -1 if no continuation on this connection is possible. 
  */
  public int closePackage( int handle, OutputStream pkg, IReceivingProtocolAdapter rpa ) throws Exception;
}
