package mwfw;

import mwfw.sync.IStorage;
import java.util.Map;
import java.util.HashMap;
import java.util.Iterator;
import java.util.ArrayList;
import java.util.NoSuchElementException;
import java.util.Dictionary;
import java.util.Hashtable;
import java.util.Enumeration;
import java.io.InputStream;
import java.io.FileOutputStream;
import java.io.FileInputStream;
import java.io.BufferedReader;
import java.io.InputStreamReader;
import java.io.PrintStream;
import java.io.ByteArrayInputStream;
import java.io.FileNotFoundException;

/**
  * Simplistic implementation of the contact database
  * Requires mwfw.pcapp.contactfile property set.
  * The value of this property is the location of the contact file.
  */
public class LocHighlightStorage implements IStorage {
  HashMap dbmetainfo = new HashMap();
  HashMap metainfo = new HashMap();	// key is the entry key, value is a Dictionary storing metainfo for the entry
  HashMap value = new HashMap();	// key is the entry key, value is a String which is the entry value
  HashMap ongoingMetaInfo = new HashMap();
  HashMap ongoingValue = new HashMap();
  ArrayList ongoingRemove = new ArrayList();
  
  public void init( HashMap initConf ) {
  }

  public void shutdown() {
  }
  
// IStorage
  public Map getDatabaseMetaInfo() {
    return (Map)dbmetainfo;
  }

  public void setDatabaseMetaInfo( Map metainf ) {
    dbmetainfo = (HashMap)metainf;
	System.err.println( "LocHighlightStorage:setDatabaseMetaInfo: "+dbmetainfo );
  }

  public Iterator selectKeys( Map queryMap ) throws Exception {
	System.err.println( "LocHighlightStorage.selectKeys: "+queryMap );
	Long timestampLimit = (Long)queryMap.get( "timestampgeq" );
	if( timestampLimit == null )
		return null;
	System.err.println( "LocHighlightStorage.selectKeys: "+timestampLimit );
    ArrayList it = new ArrayList();
    Iterator entryMetaInf = metainfo.keySet().iterator();
    for( ; entryMetaInf.hasNext() ; ) {
      String key = (String)entryMetaInf.next();
      Dictionary d = (Dictionary)metainfo.get( key );
	  String timestampString = (String)d.get( "timestamp" );
	  if( timestampString == null )
			continue;
	  long timestamp = 0L;
	  try {
		timestamp = Long.parseLong( timestampString );
	  } catch( NumberFormatException ex ) {
	    continue;
	  }
	  System.err.println( "key: "+key+",timestamp: "+timestamp );
	  if( timestamp >= timestampLimit.longValue() ) {
		it.add( key );
		System.err.println( "ADDED" );
	  }
    }
    return it.iterator();
  }
  
  public Map getEntryMetaInfo( String key ) {
    Dictionary d = (Dictionary)metainfo.get( key );
    if( d == null )
    	return null;
    return dictionaryToMap( d );
  }

  public void setEntryMetaInfo( String key, Map metainf ) {
    Dictionary d = mapToDictionary( metainf );
  }

  public InputStream getEntryData( String key ) throws Exception {
    String data = (String)value.get( key );
    if( data == null )
    	return null;
    ByteArrayInputStream bis = new ByteArrayInputStream( data.getBytes( "UTF-8" ) );
    return (InputStream)bis;
  }

  public boolean setEntryData( String key, Map metainf, InputStream data ) throws Exception {
    Dictionary d = mapToDictionary( metainf );
    StringBuffer b = new StringBuffer();
    int c = 0;
    do {
      c = data.read();
      if( c >= 0 )
      	b.append( (char)c );
    } while( c >= 0 );
    String s = new String( b );
	System.err.println( "LocHighlightStorage: setEntryData: key: "+key+
						",metainfo: "+metainf+
						",data: "+s );
    synchronized( this ) {
      ongoingMetaInfo.put( key,d );
      ongoingValue.put( key, s );
    }
	return true;
  }

  public boolean setEntryData( String key, String type, InputStream data ) throws Exception {
    boolean retval = false;
    synchronized( this ) {
      Dictionary d = (Dictionary)metainfo.get( key );
      if( d == null )
      	d = (Dictionary)new Hashtable();
      d.put( "type",type );
      String ts = new Long( System.currentTimeMillis() ).toString();
      if( d.get( "created" ) == null ) {
      	d.put( "created",ts );
      	retval = true;
      }
      d.put( "timestamp",ts );
      d.remove( "deleted" );
      Map m = dictionaryToMap( d );
      setEntryData( key,m,data );
    }
    return retval;
  }

  public void deleteEntryData( String key, Map metainf ) {
    synchronized( this ) {
      value.remove( key );
      Dictionary d = mapToDictionary( metainf );
      ongoingMetaInfo.put( key,d );
    }
  }

  public void deleteEntryData( String key ) {
    synchronized( this ) {
      ongoingRemove.add( key );
      Hashtable t = new Hashtable();
      String ts = Long.toString( System.currentTimeMillis() );
      t.put( "timestamp",ts );
      t.put( "deleted",ts );
      metainfo.put( key,t );
    }
  }

  public boolean lock( String sessionKey ) {
    return true;
  }

  public void commit( String sessionKey ) {
    synchronized( this ) {
      for( Iterator it = ongoingMetaInfo.keySet().iterator() ; it.hasNext() ; ) {
        String key = (String)it.next();
        Dictionary d = (Dictionary)ongoingMetaInfo.get( key );
        String newvalue = (String)ongoingValue.get( key );
        metainfo.put( key,d );
        value.put( key, newvalue );
      }
      ongoingMetaInfo.clear();
      ongoingValue.clear();
      for( int i = 0 ; i < ongoingRemove.size() ; ++i ) {
        String key = (String)ongoingRemove.get( i );
        value.remove( key );
      }
      ongoingRemove.clear();
    }
  }
  
// private methods
  private Map dictionaryToMap( Dictionary d ) {
    HashMap m = new HashMap();
    Enumeration keys = d.keys();
    for( ; keys.hasMoreElements() ; ) {
      Object key = keys.nextElement();
      Object value = d.get( key );
      m.put( key, value );
    }
    return m;
  }

  private Dictionary mapToDictionary( Map m ) {
    Hashtable d = new Hashtable();
    Iterator it = m.keySet().iterator();
    for( ; it.hasNext() ; ) {
      Object key = it.next();
      Object value = m.get( key );
      d.put( key, value );
    }
    return (Dictionary)d;
  }

// -------------------------- Methods called directly by MasterLocationStorage -----
/**
  * Checks the current highlight storage and deletes every item that has no
  * corresponding entry in the terms and distribution databases. The item will not
  * be actually deleted but will be turned into "tombstone" by manipulating its
  * meta-information.
  * The method assumes the database entry to be in the following format:
  * <exact longitude>:<exact latitude>:<longitude bucker>:<latitude bucket>:<description>:<address>:<terms>
  *     0				   1				2					3				4			5			6
  */
  public void cleanupObsoleteItems( LocDistroStorage ldStorage, TermsStorage tsStorage ) {
    String ts = new Long( System.currentTimeMillis() ).toString();
	for( Iterator it = value.keySet().iterator() ; it.hasNext() ; ) {
	  String key = (String)it.next();
	  String dbline = (String)value.get( key );
	  String dbItems[] = dbline.split( ":" );
	  if( dbItems.length < 7 )
		continue;
	  if( !ldStorage.matchBuckets( dbItems[2],dbItems[3] ) ||
		  !tsStorage.matchTerms( dbItems[6] ) ) {
         Dictionary d = (Dictionary)metainfo.get( key );
		 d.put( "deleted", ts );
		 d.put( "timestamp",ts );
	  }
	}
  }

/**
  * Returns whether a certain key is present in the database.
  */
  public boolean isKeyPresent( String key ) {
	return value.get( key ) != null;
  }
}
