package mwfw;

import org.xml.sax.XMLReader;
import org.xml.sax.InputSource;
import org.xml.sax.Attributes;
import org.xml.sax.SAXException;
import org.xml.sax.helpers.XMLReaderFactory;
import org.xml.sax.helpers.DefaultHandler;
import java.util.List;
import java.util.ArrayList;
import java.util.HashMap;
import java.io.FileReader;
import java.io.IOException;
import java.io.ByteArrayInputStream;

public class MasterLocationStorage extends DefaultHandler {
  static final int IN_BODY = 0;
  static final int IN_LOCATIONS = 1;
  static final int IN_LOCATION = 2;
  static final int IN_ID = 11;
  static final int IN_CENTERLONGITUDE = 3;
  static final int IN_CENTERLATITUDE = 4;
  static final int IN_EXACTLONGITUDE = 5;
  static final int IN_EXACTLATITUDE = 6;
  static final int IN_DESCRIPTION = 7;
  static final int IN_ADDRESS = 8;
  static final int IN_TERMS = 9;
  static final int IN_TERM = 10;
  int parseState;
  ArrayList locations = new ArrayList();

/**
  * Reads the location file.
  * @param locationFile Name of the location file.
  * @throw SAXException if there was a parse error.
  * Document structure: see locations.xml
  */
  public void readLocationFile( String locationFile ) throws SAXException {
    XMLReader xr = XMLReaderFactory.createXMLReader();
    xr.setContentHandler( this );
    xr.setErrorHandler( this );
	try {
	 parseState = IN_BODY;
     FileReader r = new FileReader( locationFile );
     xr.parse( new InputSource( r ) );
	} catch( IOException ex ) {
	  throw new SAXException( ex );
	}
  }

  LocationStorageEntry actualEntry = null;
  String collectString = null;

  public void startElement (String uri, String name,
                              String qName, Attributes atts) throws SAXException {
	if( qName.equals( "locations" ) ) {
	  if( parseState == IN_BODY )
		parseState = IN_LOCATIONS;
	  else
		throw new SAXException( "Misplaced <locations>" );
	} else
	if( qName.equals( "location" ) ) {
	  if( parseState == IN_LOCATIONS ) {
	    parseState = IN_LOCATION;
	    actualEntry = new LocationStorageEntry();
	  } else
		throw new SAXException( "Misplaced <location>" );
	} else
	if( qName.equals( "centerLongitude" ) ) {
	  if( parseState == IN_LOCATION ) {
		collectString = "";
		parseState = IN_CENTERLONGITUDE;
	  } else
		throw new SAXException( "Misplaced <centerLongitude>" );
	} else
	if( qName.equals( "id" ) ) {
	  if( parseState == IN_LOCATION ) {
		collectString = "";
		parseState = IN_ID;
	  } else
		throw new SAXException( "Misplaced <id>" );
	} else
	if( qName.equals( "centerLatitude" ) ) {
	  if( parseState == IN_LOCATION ) {
		collectString = "";
		parseState = IN_CENTERLATITUDE;
	  } else
		throw new SAXException( "Misplaced <centerLatitude>" );
	} else
	if( qName.equals( "exactLongitude" ) ) {
	  if( parseState == IN_LOCATION ) {
		collectString = "";
		parseState = IN_EXACTLONGITUDE;
	  } else
		throw new SAXException( "Misplaced <exactLongitude>" );
	} else
	if( qName.equals( "exactLatitude" ) ) {
	  if( parseState == IN_LOCATION ) {
		collectString = "";
		parseState = IN_EXACTLATITUDE;
	  } else
		throw new SAXException( "Misplaced <exactLatitude>" );
	} else
	if( qName.equals( "description" ) ) {
	  if( parseState == IN_LOCATION ) {
		collectString = "";
		parseState = IN_DESCRIPTION;
	  } else
		throw new SAXException( "Misplaced <description>" );
	} else
	if( qName.equals( "address" ) ) {
	  if( parseState == IN_LOCATION ) {
		collectString = "";
		parseState = IN_ADDRESS;
	  } else
		throw new SAXException( "Misplaced <address>" );
	} else
	if( qName.equals( "terms" ) ) {
	  if( parseState == IN_LOCATION ) {
		parseState = IN_TERMS;
	    actualEntry.terms = "";
	  } else
		throw new SAXException( "Misplaced <terms>" );
	} else
	if( qName.equals( "term" ) ) {
	  if( parseState == IN_TERMS ) {
		collectString = "";
		parseState = IN_TERM;
	  } else
		throw new SAXException( "Misplaced <term>" );
	} else
		throw new SAXException( "Unknown element: "+qName );
  }

  public void endElement (String uri, String name, String qName) throws SAXException {
    if ( qName.equals( "locations " ) )
		parseState = IN_BODY;
	else
	if( qName.equals( "location" ) ) {
	  if( !actualEntry.validate() )
		throw new SAXException( "current location entry is not valid" );
	  locations.add( actualEntry );
	  actualEntry = null;
	  parseState = IN_LOCATIONS;
	} else
	if( qName.equals( "id" ) ) {
	  actualEntry.id = collectString;
	  parseState = IN_LOCATION;
	} else
	if( qName.equals( "centerLongitude" ) ) {
	  actualEntry.centerLongitude = collectString;
	  parseState = IN_LOCATION;
	} else
	if( qName.equals( "centerLatitude" ) ) {
	  actualEntry.centerLatitude = collectString;
	  parseState = IN_LOCATION;
	} else
	if( qName.equals( "exactLongitude" ) ) {
	  actualEntry.exactLongitude = collectString;
	  parseState = IN_LOCATION;
	} else
	if( qName.equals( "exactLatitude" ) ) {
	  actualEntry.exactLatitude = collectString;
	  parseState = IN_LOCATION;
	} else
	if( qName.equals( "description" ) ) {
	  actualEntry.description = collectString;
	  parseState = IN_LOCATION;
	} else
	if( qName.equals( "address" ) ) {
	  actualEntry.address = collectString;
	  parseState = IN_LOCATION;
	} else
	if( qName.equals( "terms" ) ) {
	  parseState = IN_LOCATION;
	} else
	if( qName.equals( "term" ) ) {
	  if( actualEntry.terms.length() > 0 )
			actualEntry.terms += ",";
	  actualEntry.terms += collectString;
	  parseState = IN_TERMS;
	}
  }

  public void characters( char ch[], int start, int length) {
	if( ( parseState == IN_ID ) ||
		( parseState == IN_CENTERLONGITUDE ) ||
	    ( parseState == IN_CENTERLATITUDE ) ||
		( parseState == IN_EXACTLONGITUDE ) ||
		( parseState == IN_EXACTLATITUDE ) ||
		( parseState == IN_DESCRIPTION ) ||
		( parseState == IN_ADDRESS ) ||
		( parseState == IN_TERM )
	  ) {
	  String s = new String( ch, start, length );
	  collectString += s;
	}
  }

  public void updateHighlights( LocDistroStorage lsStorage, 
								LocHighlightStorage lhStorage, 
								TermsStorage tsStorage ) {
// Cleanup first highlights that don't have corresponding coordinate buckets
// or terms anymore.
	lhStorage.cleanupObsoleteItems( lsStorage, tsStorage );
// Now we iterate through the master database and find out whether all the relevant
// items in the master database are also present in the highlight database
	lhStorage.lock( "updateHighlights" );
	System.err.println( "updateHighlights" );
	for( int i = 0 ; i < locations.size() ; ++i ) {
	  LocationStorageEntry lse = (LocationStorageEntry)locations.get( i );
	  System.err.println( "lse: "+lse );
// if the key is present in the highlight database, skip.
	  if( lhStorage.isKeyPresent( lse.id ) ) {
		System.err.println( "already in lochighlight" );
		continue;
	  }
// if the entry does not fall into any coordinate bucket, skip
	  if( !lsStorage.matchBuckets( lse.centerLongitude, lse.centerLatitude ) ) {
		System.err.println( "does not match any buckets" );
		continue;
	  }
	  if( !tsStorage.matchTerms( lse.terms ) ) {
		System.err.println( "does not match any terms" );
		continue;
	  }
// We have an entry that is not yet present in the highlight database and
// fits into the current location distribution/terms profile. Add it to the
// highlights database.
// The line has the format:
// <exact longitude>:<exact latitude>:<longitude bucker>:<latitude bucket>:<description>:<address>:<terms>
	  String dbline = lse.exactLongitude + ":" +
						lse.exactLatitude + ":" +
						lse.centerLongitude + ":" +
						lse.centerLatitude + ":" +
						lse.description + ":" +
						lse.address + ":" +
						lse.terms;
	  ByteArrayInputStream is = new ByteArrayInputStream( dbline.getBytes() );
	  try {
	    lhStorage.setEntryData( lse.id, "text/plain", is );
	  } catch( Exception ex ) {}
	}
	lhStorage.commit( "updateHighlights" );
  }


// Just for testing
  public static void main( String args[] ) {
	String filename = args[0];
	MasterLocationStorage st = new MasterLocationStorage();
	try {
	  st.readLocationFile( filename );
	  for( int i = 0 ; i < st.locations.size() ; ++i ) {
		LocationStorageEntry lse = (LocationStorageEntry)st.locations.get( i );
		System.out.println( lse );
	  }
	} catch( Exception ex ) {
	  ex.printStackTrace();
	}
  }

  class LocationStorageEntry {
    public String id = null;
    public String centerLongitude = null;
    public String centerLatitude = null;
	public String exactLongitude = null;
	public String exactLatitude = null;
	public String terms = null;
    public String description = null;
	public String address = null;

/**
  * Validates whether the object is correctly filled.
  * @return true if the object is correctly filled
  */
    public boolean validate() {
	  return ( centerLongitude != null ) &&
			 ( centerLatitude != null ) &&
			 ( exactLongitude != null ) &&
			 ( exactLatitude != null ) &&
			 ( terms != null ) &&	
			 ( description != null ) &&
			 ( address != null );
    }

    public String toString() {
	  StringBuffer sb = new StringBuffer();
	  sb.append( "id: "+id );
	  sb.append( ",centerLongitude: "+centerLongitude );
	  sb.append( ",centerLatitude: "+centerLatitude );
	  sb.append( ",exactLongitude: "+exactLongitude );
	  sb.append( ",exactLatitude: "+exactLatitude );
	  sb.append( ",description: "+description );
	  sb.append( ",address: "+address );
	  sb.append( ",search terms: "+terms );
	  return new String( sb );
	}
  }

}
