package mwfw.common;

import mwfw.ISendingProtocolAdapter;
import mwfw.IReceivingProtocolAdapter;
import mwfw.IReceivingDispatcher;
import mwfw.MasterLocationStorage;
import mwfw.LocDistroStorage;
import mwfw.LocHighlightStorage;
import mwfw.TermsStorage;
import mwfw.syncmlds.SyncServer;
import mwfw.sync.TimeStampSyncEngine;
import mwfw.sync.ISyncListener;
import mwfw.syncmlds.impl.Base64;

import java.io.OutputStream;
import java.io.InputStream;
import java.io.IOException;
import java.net.URL;
import java.net.URLConnection;
import java.util.HashMap;
import java.util.Dictionary;
import javax.servlet.Servlet;
import javax.servlet.ServletException;
import javax.servlet.ServletRequest;
import javax.servlet.ServletResponse;
import javax.servlet.ServletConfig;
import javax.servlet.http.HttpServletRequest;

class ConnMapEntry {
  public ConnMapEntry( URLConnection conn, String address ) {
    this.conn = conn;
    this.address = address;
  }

  public URLConnection conn;
  public String address;
}

class SyncEngineConfiguration implements ISyncListener {
  public SyncServer syncServer;
  public ISendingProtocolAdapter spa;
  public TimeStampSyncEngine tsSyncEngine;
  public LocDistroStorage lsStorage;
  public LocHighlightStorage lhStorage;
  public TermsStorage tsStorage;
  public MasterLocationStorage masterLocationStorage;

  public void setSendingProtocolAdapter( ISendingProtocolAdapter spa ) {
	this.spa = spa;
  }

  public void setMasterLocationStorage( MasterLocationStorage masterLocationStorage ) {
	this.masterLocationStorage = masterLocationStorage;
  }

  public void createEngine() {
	syncServer = new SyncServer();
	tsSyncEngine = new TimeStampSyncEngine();
	lsStorage = new LocDistroStorage();
	lhStorage = new LocHighlightStorage();
	tsStorage = new TermsStorage();
	syncServer.setSendingProtocolAdapter( spa );
	syncServer.setSyncProtocolAdapterCallBack( tsSyncEngine );
	tsSyncEngine.setSyncProtocolAdapter( syncServer );
	tsSyncEngine.setStorage( "locdistro",lsStorage );
	tsSyncEngine.setStorage( "lochighlight",lhStorage );
	tsSyncEngine.setStorage( "terms",tsStorage );
	tsSyncEngine.addSyncListener( this );
	lsStorage.init( null );
	syncServer.init( null );
	HashMap tsInit = new HashMap();
	tsInit.put( "mwfw.sync.timestamp.source","http://localhost:8080/peebleweb/SyncServer" );
	tsSyncEngine.init( tsInit );
  }

  public void sessionStarted( String sessionKey, String target_db, boolean client ) {
	System.err.println( "sessionStarted: "+target_db );
  }

  public void sessionEnded( String sessionKey, String target_db, boolean client ) {
	System.err.println( "sessionEnded: "+target_db );
	if( target_db.equals( "terms" ) )
		masterLocationStorage.updateHighlights( lsStorage, lhStorage, tsStorage );
  }
}


/**
  * Component implementing HTTP-based protocol adapter.
  */
public class HttpProtocolAdapter implements 
					ISendingProtocolAdapter,
 					Servlet {
  HashMap connmap = new HashMap();
  HashMap incomingconnmap = new HashMap();
  int handlectr = 0;
  HashMap<String,SyncEngineConfiguration> engineMap = new HashMap<String,SyncEngineConfiguration>();
  MasterLocationStorage masterLocationStorage;

  public void init( HashMap initConf ) {
  }

  public void shutdown() {
  }

// ------------------ ISendingProtocolAdapter
  public OutputStream sendPackage( int handle, String address ) 
														throws Exception {
// This implementation may receive existing connections only if the connection was
// initiated by an incoming request. In this case the output stream must be present
// in incomingconnmap.
    if( handle != -1 ) {
      OutputStream os = (OutputStream)incomingconnmap.get( (Object)new Integer( handle ) );
      if( os == null )
      	throw new IOException( "os == null in HttpProtocolAdapter:sendPackage" );
      return os;
	}
    URL url = new URL( address );
    URLConnection urlc = url.openConnection();
    urlc.setDoInput( true );
    urlc.setDoOutput( true );
    urlc.setUseCaches( true );
    urlc.setRequestProperty( "Content-Type","application/octet-stream" );
    OutputStream os = urlc.getOutputStream();
    ConnMapEntry cme = new ConnMapEntry( urlc, address );
    connmap.put( (Object)os,(Object)cme );
    return os;
  }

  public int closePackage( int handle, OutputStream pkg, IReceivingProtocolAdapter rpa ) throws Exception {
    pkg.flush();
 // In this case this was an incoming connection and the connection is closed in the service() method.
    if( handle != -1 ) {
      return -1;
    }
    URLConnection urlc = null;
    String address = null;
    synchronized( connmap ) {
      ConnMapEntry cme = (ConnMapEntry)connmap.get( (Object)pkg );
      urlc = cme.conn;
      address = cme.address;
      if( urlc == null )
		throw new IOException( "urlc == null in HttpProtocolAdapter:closePackage" ); 
      connmap.remove( (Object)pkg );
    }
    boolean handled = false;
    InputStream is = urlc.getInputStream();
    if( rpa != null )
      handled = rpa.receivePackage( -1, address, is );
    if( !handled )
		is.close();
	return -1; 
  }

// -------------------- Servlet
  private ServletConfig servletConfig = null; 

  public void init(ServletConfig config) throws ServletException {
    this.servletConfig = config;
	masterLocationStorage = new MasterLocationStorage();
	String locationFile = config.getInitParameter( "locationFileName" );
    try {
	  if( locationFile != null )
		masterLocationStorage.readLocationFile( locationFile );
	} catch( Exception ex ) {
	  throw new ServletException( ex );
	}
  }
  
  public ServletConfig getServletConfig() {
    return servletConfig;
  }
  
  public String getServletInfo() {
    return "Peeble SyncServer";
  }

  public void destroy() {
    this.servletConfig = null;
  }

  public void service( ServletRequest req, ServletResponse res ) throws IOException,ServletException {
    int thishandle = handlectr++;
    InputStream is = req.getInputStream();
    res.setContentType( "application/octet-stream" );
	String authHeader = ((HttpServletRequest)req).getHeader( "Authorization" );
	String username = null;
	String password = null;
	if( authHeader != null ) {
	  if( authHeader.startsWith( "Basic" ) && ( authHeader.length() > 6 ) ) {
		String credString = new String( Base64.decode( authHeader.substring( 6 ) ) );
		String creds[] = credString.split( ":" );
		if( creds.length == 2 ) {
		  username = creds[0];
		  password = creds[1];
		}
	  }
	} 
	if( username == null )
		username = "<none>";
	SyncEngineConfiguration conf = engineMap.get( username );
	if( conf == null ) {
	  conf = new SyncEngineConfiguration();
	  conf.setSendingProtocolAdapter( this );
	  conf.setMasterLocationStorage( masterLocationStorage );
	  conf.createEngine();
	  engineMap.put( username,conf );
	}
    OutputStream os = res.getOutputStream();
    Integer thisHandle = new Integer( thishandle );
    synchronized( incomingconnmap ) {
      incomingconnmap.put( (Object)thisHandle,(Object)os );
    }
    try {
      conf.syncServer.receivePackage( thishandle,null,is );
    } catch( Exception ex ) {
      ex.printStackTrace();
      throw new ServletException( ex );
    }
    is.close();
    os.close();
    synchronized( incomingconnmap ) {
      incomingconnmap.remove( (Object)thisHandle );
    }
  }
}
