package mwfw.syncmlds;

import mwfw.syncmlds.impl.SyncMLUtil;
import mwfw.syncmlds.impl.ParserState;
import mwfw.syncmlds.impl.IReceiveCommand;
import mwfw.syncmlds.impl.SyncSession;
import mwfw.syncmlds.impl.QueryHandle;

import mwfw.ISendingProtocolAdapter;
import mwfw.IReceivingProtocolAdapter;
import mwfw.sync.ISyncProtocolAdapter;
import mwfw.sync.ISyncProtocolAdapterCallBack;
import mwfw.syncmlds.impl.SyncSession;
import javax.xml.parsers.SAXParserFactory;
import javax.xml.parsers.SAXParser;
import org.xml.sax.SAXException;
import org.xml.sax.XMLReader;
import org.xml.sax.InputSource;

import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.io.ByteArrayInputStream;
import java.io.PrintStream;
import java.io.UnsupportedEncodingException;
import java.util.HashMap;
import java.util.ArrayList;
import java.util.Map;
import java.util.List;
import java.util.HashMap;
import java.util.Iterator;


/**
  * SyncML DS implementation server component.
  */
public class SyncServer implements ISyncProtocolAdapter,IReceiveCommand,IReceivingProtocolAdapter {
  ISendingProtocolAdapter spa = null;
  ISyncProtocolAdapterCallBack pacb = null;
  SAXParserFactory pf = null;
  HashMap sessions = new HashMap();
  int queryhandles = 0;
  HashMap queries = new HashMap();
  
  public void init( HashMap initConf ) {
    this.pf = SAXParserFactory.newInstance();
    sessions.clear();
  }

  public void shutdown() {
    this.pf = null;
  }

  public void setSendingProtocolAdapter( ISendingProtocolAdapter spa ) {
    this.spa = spa;
  }
  
  public void unsetSendingProtocolAdapter( ISendingProtocolAdapter spa ) {
    this.spa = null;
  }

  public void setSyncProtocolAdapterCallBack( ISyncProtocolAdapterCallBack pacb ) {
    this.pacb = pacb;
  }

  public void unsetSyncProtocolAdapterCallBack( ISyncProtocolAdapterCallBack pacb ) {
    this.pacb = null;
  }

// ISyncProtocolAdapter
/**
  * openSession for SyncML DS server always throws an exception: the session is
  * opened by the SyncML DS client.
  */
  public int openSession( String peer, Map syncparams, ISyncProtocolAdapterCallBack cb ) throws Exception {
    throw new RuntimeException( "SyncML server cannot open session" ); 
  }

  public void abortSession( int handle ) {
  }

  public void suspend() {
  }
  
  public void resume() {
  }

  public int sendQuery( String sessionKey, Map metainf, String key ) throws Exception {
    SyncSession session = (SyncSession)sessions.get( sessionKey );
    if( session == null )
    	throw new SAXException( "no session object for session key "+sessionKey );
    String type = (String)metainf.get( "type" );
    int qid = queryhandles++;
    Integer qidi = new Integer( qid );
    QueryHandle qh = new QueryHandle();
    qh.cmdID = session.rspCmdIDCtr;
    qh.msgID = session.lastMsgID;
    qh.sessionKey = sessionKey;
    qh.handle = qid;
    SyncMLUtil.sendGetCommand( session.rspCmdIDCtr, type, key, session.rspStream );
  	++session.rspCmdIDCtr;
  	synchronized( queries ) {
  	  queries.put( qidi,qh );
  	}
  	return qid;
  }
  
  public void sendAddition( String sessionKey, Map metinf, String key, InputStream is ) throws Exception {
    SyncSession session = (SyncSession)sessions.get( sessionKey );
    if( session == null )
    	throw new SAXException( "no session object for session key "+sessionKey );
    String type = (String)metinf.get( "type" );
    SyncMLUtil.sendAddCommand( session.rspCmdIDCtr, key,  
  							type, is, session.rspStream );
  	++session.rspCmdIDCtr;
  }

  public void sendReplacement( String sessionKey, Map metinf, String key, InputStream is ) throws Exception {
    SyncSession session = (SyncSession)sessions.get( sessionKey );
    if( session == null )
    	throw new SAXException( "no session object for session key "+sessionKey );
    String type = (String)metinf.get( "type" );
    SyncMLUtil.sendReplaceCommand( session.rspCmdIDCtr, key,  
  							type, is, session.rspStream );
  	++session.rspCmdIDCtr;
  }

  public void sendDeletion( String sessionKey, String key ) throws Exception {
    SyncSession session = (SyncSession)sessions.get( sessionKey );
    if( session == null )
    	throw new SAXException( "no session object for session key "+sessionKey );
    SyncMLUtil.sendDeleteCommand( session.rspCmdIDCtr, key, session.rspStream ); 
  	++session.rspCmdIDCtr;
  }

// IReceivingProtocolAdapter
  public boolean receivePackage( int handle, String address, InputStream pkg ) throws Exception {
    ArrayList response = new ArrayList();
    ParserState s = new ParserState( (IReceiveCommand)this,true );
    SAXParser saxParser = pf.newSAXParser();
    XMLReader parser = saxParser.getXMLReader();
    OutputStream os = spa.sendPackage( handle, address );
    s.rspStream = os;
    parser.setContentHandler( s );
    InputSource src = new InputSource( pkg );
    src.setEncoding( "utf-8" );
    parser.parse( src );
    SyncSession session = s.session;
    PrintStream ps = session.rspStream;
    switch( s.msgID ) {
      case 1:
        String misc = "<Meta><Anchor xmlns='syncml:metinf'><Last>"+session.last+"</Last>" +
        			  "<Next>"+session.next+"</Next></Anchor></Meta>";
      	SyncMLUtil.sendAlert( session.rspCmdIDCtr++, session.type, session.target_dbname, 
      						  session.source_dbname, misc, ps );
      	break;
      	
      case 2:
        ps.println( "<Sync><CmdID>"+session.rspCmdIDCtr+"</CmdID>" );
  	    ++session.rspCmdIDCtr;
        ps.println( "<Target>"+session.target_dbname+"</Target><Source>"+session.source_dbname+"</Source>" );
        if( pacb != null )
        	pacb.sendCommands( session.sessionKey,(ISyncProtocolAdapter)this );
        ps.println( "</Sync>" );
        break; 
    }
    ps.println( "<Final/></SyncBody></SyncML>" );
    spa.closePackage( handle, os, null );
// This was the last message of the session, remove it
    if( s.msgID == 3 ) {
      synchronized( sessions ) {
        sessions.remove( session );
        if( pacb != null )
        	pacb.endSession( session.sessionKey );
      }
    }
    return true;
  }

// IReceiveCommand
  public void endSyncHdr( ParserState parser ) throws SAXException {
    String sessionKey = parser.sessionKey;
    SyncSession session = null;
    int msgID = parser.msgID;
    if( msgID == 1 ) {
      session = new SyncSession();
      session.sessionKey = sessionKey;
      session.source = parser.source;
      session.target = parser.target;
      session.lastMsgID = 1;
      session.sessionID = parser.sessionID;
      session.timestamp = System.currentTimeMillis();
      synchronized( sessions ) {
        sessions.put( sessionKey,session );
      }
      parser.session = session;
    } else {
      session = (SyncSession)sessions.get( sessionKey );
  	  if( session == null )
  		throw new SAXException( "no session for session key "+sessionKey );
      session.timestamp = System.currentTimeMillis();
      if( parser.msgID != ( session.lastMsgID+1 ) )
      	throw new SAXException( "On session "+sessionKey+" last message seen: "+session.lastMsgID+
      											" message arrived: "+parser.msgID );
      session.lastMsgID = parser.msgID;
      parser.session = session;
    }
    PrintStream ps = null;
    try {
      ps = new PrintStream( parser.rspStream,true,"UTF-8" );
    } catch( UnsupportedEncodingException ex ) {
      throw new SAXException( ex );
    }
    session.rspStream = ps;
    session.rspCmdIDCtr = 1;
    ps.println( "<SyncML>" );
    SyncMLUtil.sendResponseHeader( session,ps );
    ps.println( "<SyncBody>" );
    SyncMLUtil.sendStatus( session.rspCmdIDCtr++, session.lastMsgID, 
    					  0, "SyncHdr", session.source, session.target,
  								212, null,ps ); 
    if( ParserState.debug )
    	System.out.println( "SyncServer:endSyncHdr: sessionKey: "+sessionKey+" msgID: "+msgID );
  }

  public void dbAlertInPackage1( String sessionKey, int cmdID, int type, String source, String target, 
  							long last, long next ) throws SAXException {
  	SyncSession session = (SyncSession)sessions.get( sessionKey );
    PrintStream ps = session.rspStream;
  	if( session == null )
  		throw new SAXException( "no session for session key "+sessionKey );
    session.timestamp = System.currentTimeMillis();
    session.type = type;
    session.source_dbname = source;
    session.target_dbname = target;
    session.last = last;
    session.next = next;
    if( pacb != null ) {
// This syncprotadapter uses the following keys:
// source_db: source:source_dbname
// target_db: target_dbname
// last: Long( last )
// next: Long( next )
      HashMap metainf = new HashMap();
      metainf.put( "source", session.source );
      metainf.put( "target", session.target );
      metainf.put( "source_db",session.source_dbname );
      metainf.put( "target_db",session.target_dbname );
      metainf.put( "last",new Long( last ) );
      metainf.put( "next",new Long( next ) );
      try {
        pacb.syncMetaInfo( sessionKey,metainf,false );
      } catch( Exception ex ) {
        ex.printStackTrace();
        throw new SAXException( ex );
      }
      SyncMLUtil.sendResponseHeader( session,ps );
    }
    String statusitem = "<Item><Data><Anchor xmlns='syncml:metinf'><Next>"+next+"</Next></Anchor></Data></Item>";
    SyncMLUtil.sendStatus( session.rspCmdIDCtr++, session.lastMsgID, 
    					  cmdID, "Alert", session.source, session.target,
  								200, statusitem,ps ); 
    if( ParserState.debug )
    	System.out.println( "SyncServer:dbAlertInPackage1: sessionKey: "+sessionKey+" cmdID: "+cmdID+" type: "+type+
    					" Source: "+source+" Target: "+target+" last: "+last+" next: "+next ); 
  }

  public void putInPackage1( String sessionKey, int cmdID, String devinf ) throws SAXException {
  	SyncSession session = (SyncSession)sessions.get( sessionKey );
    PrintStream ps = session.rspStream;
  	if( session == null )
  		throw new SAXException( "no session for session key "+sessionKey );
    session.timestamp = System.currentTimeMillis();
    SyncMLUtil.sendStatus( session.rspCmdIDCtr++, session.lastMsgID, 
    					  cmdID, "Put", "./devinf11", null,
  								200, null,ps );     
    if( ParserState.debug )
    	System.out.println( "SyncServer:putInPackage1: sessionKey: "+sessionKey+" cmdID: "+cmdID+" devinf: "+devinf ); 
  }

  public void getCommand( String sessionKey, int msgID, int cmdID, String type, String target ) throws SAXException {
  	SyncSession session = (SyncSession)sessions.get( sessionKey );
  	if( session == null )
  		throw new SAXException( "no session for session key "+sessionKey );
    session.timestamp = System.currentTimeMillis();
    PrintStream ps = session.rspStream;
    if( target.equals( "./devinf11" ) ) {
      String devinf = "<DevInf xmlns='syncml:devinf'><SwV>0.0.1</SwV></DevInf>";
      SyncMLUtil.sendResults( session.rspCmdIDCtr++, session.lastMsgID, 
      				cmdID, "application/vnd.syncml-devinf+xml", "./devinf11", 
  									null, devinf, ps );
    } else
    if( pacb != null ) {
      InputStream is = null;
      try {
        if( pacb != null ) {
          is = pacb.syncGet( sessionKey, null, target );
          SyncMLUtil.sendResults( session.rspCmdIDCtr++, session.lastMsgID, 
      				cmdID, type, target, is, null, ps );
        }
      } catch( Exception ex ) {
        throw new SAXException( ex );
      }
    }
    if( ParserState.debug ) 
    	System.out.println( "SyncServer:getCommand: sessionKey: "+sessionKey+" msgID: "+msgID+
    					" cmdID: "+cmdID+" type: "+type+" target: "+target ); 
  }

  public void statusCommand( String sessionKey, int msgID, int cmdID, int data, int msgRef, int cmdRef, 
  							String cmd, String sourceRef, String targetRef ) {}

  public void resultsCommand( String sessionKey, int msgID, int cmdID, int msgRef, int cmdRef, 
      		  							String results_Source, String results_Type, String results_Data ) 
      		  									throws SAXException {
    if( !results_Source.equals( "./devinf11" ) ) {
      SyncSession session = (SyncSession)sessions.get( sessionKey );
      if( session == null )
    	throw new SAXException( "no session object for session key "+sessionKey );
      HashMap m = new HashMap();
      m.put( "type",results_Type );
      int qid = -1;
      synchronized( queries ) {
        for( Iterator i = queries.entrySet().iterator() ; i.hasNext() ; ) {
          QueryHandle qh = (QueryHandle)i.next();
          if( sessionKey.equals( qh.sessionKey ) &&
              ( cmdRef == qh.cmdID ) &&
              ( msgRef == qh.msgID ) ) {
            qid = qh.handle;
            i.remove();
            break;
          }
        }
      }
      ByteArrayInputStream is = new ByteArrayInputStream( results_Data.getBytes() );
      try {
      	if( pacb != null )
        	pacb.syncQueryResult( sessionKey, qid, m, results_Source, is );
      } catch( Exception ex ) {
        throw new SAXException( ex );
      }
    }
  }

  public void addReplaceCommand( String sessionKey, boolean replace, int msgID, int cmdID,  
  										String source_db,String target_db,
      		  							String source, String type, 
      		  							String data ) throws SAXException {
    SyncSession session = (SyncSession)sessions.get( sessionKey );
    if( session == null )
    	throw new SAXException( "no session object for session key "+sessionKey );
    HashMap m = new HashMap();
    m.put( "type",type );
    ByteArrayInputStream is = new ByteArrayInputStream( data.getBytes() );
    try {
      if( pacb != null )
      	pacb.syncAddReplace( sessionKey, replace, m, source_db, target_db, source, is );
      String cmdName = replace ? "Replace" : "Add";
  	  SyncMLUtil.sendStatus( session.rspCmdIDCtr, msgID, cmdID, cmdName,
  								session.target, session.source,
  								200, null, session.rspStream );
  	  ++session.rspCmdIDCtr;
    } catch( Exception ex ) {
      ex.printStackTrace();
      throw new SAXException( ex );
    }
  }

  public void deleteCommand( String sessionKey, int msgID, int cmdID, String target_db, String target )
  							throws SAXException {
    SyncSession session = (SyncSession)sessions.get( sessionKey );
    if( session == null )
    	throw new SAXException( "no session object for session key "+sessionKey );
    ISyncProtocolAdapterCallBack cb = session.cb;
    try {
      if( pacb != null )
      	pacb.syncDelete( sessionKey, target_db, target );
  	  SyncMLUtil.sendStatus( session.rspCmdIDCtr, msgID, cmdID, "Delete",
  								session.target, session.source,
  								200, null, session.rspStream );
  	  ++session.rspCmdIDCtr;
    } catch( Exception ex ) {
      throw new SAXException( ex );
    }    
  }

  public void syncCommand( String sessionKey, int msgID, int cmdID, String source_db, String target_db )
  							throws SAXException {
    SyncSession session = (SyncSession)sessions.get( sessionKey );
    if( session == null )
    	throw new SAXException( "no session object for session key "+sessionKey );
  	SyncMLUtil.sendStatus( session.rspCmdIDCtr, msgID, cmdID, "Sync",
  								session.target, session.source,
  								200, null, session.rspStream );
  	++session.rspCmdIDCtr;
  }

  public void endMessage( String sessionKey, int msgID ) {}
}
