package mwfw;

import java.io.InputStream;

/**
  * This interface is implemented by components wishing to receive incoming packages.
  * These components expose this interface and the protocol adapter component connects
  * to these components and delivers the packages to the input package listeners.
  * Components implementing this interface may specify mwfw.address.filter property. If
  * this property exists, only packages whose address starts with the property value are
  * delivered to the receivers.
  */ 
public interface IReceivingProtocolAdapter {
/**
  * Called by the protocol adapter if there is an incoming package.
  * @param handle Handle number if this package is associated with some existing connection
  *               or -1 if the bearer is connectionless or no previous connection exists.
  * @param address Package originator's address in URI format.
  * @param pkg The incoming package. This can be a real stream or a package of fixed
  *            length.
  * @return true if this package was handled by this protocol adapter, false otherwise.
  */
  public boolean receivePackage( int handle, String address, InputStream pkg ) throws Exception;
}
 