package mwfw;

import mwfw.common.HttpProtocolAdapter;
import mwfw.sync.TimeStampSyncEngine;
import mwfw.sync.ISyncProtocolAdapter;
import mwfw.sync.IStorage;
import mwfw.sync.ISyncListener;
import mwfw.syncmlds.SyncClient;

import java.util.HashMap;
import java.util.concurrent.Semaphore;
import java.io.ByteArrayInputStream;

public class TestClient {
  private static final String SYNCDB_LOCDISTRO_NAME = "locdistro";
  private static final String SYNCDB_LOCHIGHLIGHT_NAME = "lochighlight";
  private static final String SYNCDB_TERMS_NAME = "terms";
  Semaphore terminate = new Semaphore( 0 );

  public void run( String args[] ) {
	String targetAddress = 
	"http://localhost:8080/peebleweb/SyncServer?username=testuser&password=testpassword";
    HttpProtocolAdapter pa = new HttpProtocolAdapter();
	SyncClient sc = new SyncClient();
	TimeStampSyncEngine ts = new TimeStampSyncEngine();
	LocDistroStorage ssa = new LocDistroStorage();
	fillTestDatabase1( ssa );
	LocHighlightStorage ssh = new LocHighlightStorage();
	TermsStorage sst = new TermsStorage();
	fillTermsTestDatabase( sst );
	sc.setSendingProtocolAdapter( (ISendingProtocolAdapter)pa );
	ts.setSyncProtocolAdapter( (ISyncProtocolAdapter)sc );
	ts.setStorage( SYNCDB_LOCDISTRO_NAME, (IStorage)ssa );
	ts.setStorage( SYNCDB_LOCHIGHLIGHT_NAME, (IStorage)ssh );
	ts.setStorage( SYNCDB_TERMS_NAME, (IStorage)sst );
    SyncListener sl = new SyncListener();
	ts.addSyncListener( (ISyncListener)sl );
	pa.init();
	sc.init();
	ssa.init( null );
	HashMap tsInit = new HashMap();
	tsInit.put( "mwfw.sync.timestamp.source","imei:000000" );
	ts.init( tsInit );
	System.out.println( "Sync round #1" );
    try {
	  ts.startSync( targetAddress,
					SYNCDB_LOCDISTRO_NAME );
	  terminate.acquire();
    } catch( Exception ex ) {
	  System.out.println( "Synchronization error" );
	  ex.printStackTrace();
      return;
    }
	System.out.println( "Sync round #2" );
	fillTestDatabase2( ssa );
    try {
	  ts.startSync( targetAddress,
					SYNCDB_LOCDISTRO_NAME );
	  terminate.acquire();
    } catch( Exception ex ) {
	  System.out.println( "Synchronization error" );
	  ex.printStackTrace();
      return;
    }
	System.out.println( "Synchronizing terms" );
    try {
	  ts.startSync( targetAddress,
					SYNCDB_TERMS_NAME );
	  terminate.acquire();
    } catch( Exception ex ) {
	  System.out.println( "Synchronization error" );
	  ex.printStackTrace();
      return;
    }
	System.out.println( "Synchronizing highlights" );
    try {
	  ts.startSync( targetAddress,
					SYNCDB_LOCHIGHLIGHT_NAME );
	  terminate.acquire();
    } catch( Exception ex ) {
	  System.out.println( "Synchronization error" );
	  ex.printStackTrace();
      return;
    }
  }

  public static void main( String args[] ) {
	TestClient c = new TestClient();
	c.run( args );
  }

  class SyncListener implements ISyncListener {
	  public void sessionStarted( String sessionKey,boolean client ) {
		System.out.println( "Sync session started: "+sessionKey );
	  }

	  public void sessionEnded( String sessionKey,boolean client ) {
		System.out.println( "Sync session ended: "+sessionKey );	
		terminate.release();
	  }
  }

  private void fillTestDatabase1( IStorage ssa ) {
	long baseTimeStamp = System.currentTimeMillis();
	long entry1Time = baseTimeStamp - 10000L;
	HashMap entry1MI = new HashMap();
	entry1MI.put( "created", Long.toString( entry1Time ) );
    entry1MI.put( "timestamp",Long.toString( entry1Time ) );
    entry1MI.put( "type","text/plain" );
	String entry1Data = "13.907,46.385,0.4";
	ByteArrayInputStream entry1IS = new ByteArrayInputStream( entry1Data.getBytes() );
	try {
	  ssa.setEntryData( "1",entry1MI,entry1IS );
	  ssa.commit( null );
	} catch( Exception ex ) {
	  ex.printStackTrace();
	}
  }

  private void fillTestDatabase2( IStorage ssa ) {
	long baseTimeStamp = System.currentTimeMillis();
	long entry2Time = baseTimeStamp;
	HashMap entry2MI = new HashMap();
	entry2MI.put( "created", Long.toString( entry2Time ) );
    entry2MI.put( "timestamp",Long.toString( entry2Time ) );
    entry2MI.put( "type","text/plain" );
	String entry2Data = "13.907,46.384,1.1";
	ByteArrayInputStream entry2IS = new ByteArrayInputStream( entry2Data.getBytes() );
	try {
	  ssa.setEntryData( "2",entry2MI,entry2IS );
	  ssa.commit( null );
	} catch( Exception ex ) {
	  ex.printStackTrace();
	}
  }

  private void fillTermsTestDatabase( IStorage sst ) {
	long baseTimeStamp = System.currentTimeMillis();
	long termTime = baseTimeStamp;
	HashMap termMI = new HashMap();
	termMI.put( "created", Long.toString( termTime ) );
    termMI.put( "timestamp",Long.toString( termTime ) );
    termMI.put( "type","text/plain" );
	String termData = "wine,shop";
	ByteArrayInputStream termIS = new ByteArrayInputStream( termData.getBytes() );
	try {
	  sst.setEntryData( "1",termMI,termIS );
	  sst.commit( null );
	} catch( Exception ex ) {
	  ex.printStackTrace();
	}
  }

}
