package mwfw.syncmlds;

import mwfw.ISendingProtocolAdapter;
import mwfw.IReceivingProtocolAdapter;
import mwfw.sync.ISyncProtocolAdapter;
import mwfw.sync.ISyncProtocolAdapterCallBack;
import mwfw.syncmlds.impl.SyncSession;
import mwfw.syncmlds.impl.SyncMLUtil;
import mwfw.syncmlds.impl.QueryHandle;
import mwfw.syncmlds.impl.IReceiveCommand;
import mwfw.syncmlds.impl.ParserState;
import mwfw.syncmlds.impl.Semaphore;

import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.io.PrintStream;
import java.io.ByteArrayOutputStream;
import java.io.ByteArrayInputStream;
import java.io.UnsupportedEncodingException;
import java.util.Map;
import java.util.HashMap;
import java.util.List;
import java.util.Iterator;
import javax.xml.parsers.SAXParserFactory;
import javax.xml.parsers.SAXParser;
import org.xml.sax.SAXException;
import org.xml.sax.XMLReader;
import org.xml.sax.InputSource;

/**
  * SyncML DS implementation client component.
  */
public class SyncClient implements ISyncProtocolAdapter,IReceivingProtocolAdapter,IReceiveCommand {
  ISendingProtocolAdapter spa = null;
  int sessctr = 0;
  SAXParserFactory pf = null;
  HashMap sessions = new HashMap();
  int queryhandles = 0;
  HashMap queries = new HashMap();
  static final boolean debug = false;
  
  public void init() {
    this.pf = SAXParserFactory.newInstance();
  }

  public void shutdown() {
    this.pf = null;
    sessions.clear();
    queries.clear();
  }

  public void setSendingProtocolAdapter( ISendingProtocolAdapter spa ) {
    this.spa = spa;
  }
  
  public void unsetSendingProtocolAdapter() {
    this.spa = null;
  }

// ISyncProtocolAdapter
/**
  * openSession for SyncML DS supports the following syncparams:
  * <ul>
  * <li>dbname: URI of the database to synchronize. If only this parameter is present, the source and
  *     the target database will be the same.
  * <li>target_dbname: URI of the target database. This must be present together with dbname, in this
  *     case dbname is the URI of the source database and target_dbname is the target database.
  * <li>source: network ID of this device in URI format
  * <li>username: user ID for authentication on the other device (no authentication if omitted)
  * <li>password: password for authentication on the other device.
  * <li>maxmsgsize: Value for MaxMsgSize meta element (element will not be present if not set)
  * <li>last: timestamp of the last time this database was synchronized with this peer (Long object value)
  * <li>next: timestamp of this synchronization session
  * </ul>
  */
  public int openSession( String origPeer, Map syncparams, ISyncProtocolAdapterCallBack cb ) throws Exception {
    if( spa == null )
    	throw new IOException( "SyncClient.openSession: sending protocol adapter not set" );
    String dbname = (String)syncparams.get( "dbname" );
    if( dbname == null )
    	throw new RuntimeException( "SyncClient.openSession: dbname sync param not set" );
    String target_dbname = (String)syncparams.get( "target_dbname" );
    if( target_dbname == null )
    	target_dbname = dbname;
    OutputStream os = spa.sendPackage( -1, origPeer );
	String peer = origPeer;
	int qmidx = origPeer.indexOf( '?' );
	if( qmidx >= 0 )
		peer = origPeer.substring( 0,qmidx );
    PrintStream ps = new PrintStream( os,true,"UTF-8" );
    ps.println( "<SyncML>" );
    int newsession = sessctr++;
    String source = (String)syncparams.get( "source" );
    String sessionKey = ParserState.getSessionKey( peer,newsession );
    SyncSession session = new SyncSession();
    session.source = source;
    session.target = peer;
    session.source_dbname = dbname;
    session.target_dbname = target_dbname;
    session.sessionKey = sessionKey;
    Semaphore s = new Semaphore();
    session.notifySemaphore = s;
    session.cb = cb;
	if( debug ) {
	  System.out.println( "New session; source: "+session.source+
							",target: "+session.target+
							",source_dbname: "+session.source_dbname+
							"target_dbname: "+session.target_dbname+
							"sessionKey: "+session.sessionKey );
	}
    synchronized( sessions ) {
      sessions.put( sessionKey,session );
    }
    if( source == null )
    	throw new RuntimeException( "source parameter not set for SyncClient:openSession" );
    Long last = (Long)syncparams.get( "last" );
    if( last == null )
    	throw new RuntimeException( "last parameter not set for SyncClient:openSession" );
    Long next = (Long)syncparams.get( "next" );
    if( next == null )
    	throw new RuntimeException( "next parameter not set for SyncClient:openSession" );
// Msg#1
    SyncMLUtil.sendSyncMLHeader( newsession, 1, source, peer, syncparams, ps );
    ps.println( "<SyncBody>" );
    SyncMLUtil.sendDbAlert( 1, SyncMLUtil.SYNCTYPE_TWO_WAY,
  									last.longValue(),next.longValue(), 
  									dbname,target_dbname,ps );
  	if( cb != null ) {
  	  HashMap syncmetainf = new HashMap();
  	  syncmetainf.put( "source", session.source );
      syncmetainf.put( "target", session.target );
      syncmetainf.put( "source_db",session.source_dbname );
      syncmetainf.put( "target_db",session.target_dbname );
      syncmetainf.put( "last",last );
      syncmetainf.put( "next",next );
      try {
        cb.syncMetaInfo( sessionKey,syncmetainf,true );
      } catch( Exception ex ) {
        ex.printStackTrace();
        throw new SAXException( ex );
      }
  	}
  	HashMap m = new HashMap();
  	m.put( (Object)"SwV",(Object)"0.0.1" );
  	SyncMLUtil.sendDevInfo( 2, (Map)m, ps );
    ps.println( "<Final/></SyncBody></SyncML>" );
    spa.closePackage( -1, os,(IReceivingProtocolAdapter)this );
    s.p();
// Msg#2
// Send session header generated for the response message first
    s.reset();
    os = spa.sendPackage( -1, origPeer );
    ps = new PrintStream( os,true,"UTF-8" );
    ps.println( "<SyncML>" );
    session.lastMsgID++;
    ps.print( session.bos.toString( "utf-8" ) );
    session.bos = null;
    session.rspStream = ps;
    ps.println( "<Sync><CmdID>"+session.rspCmdIDCtr+"</CmdID>" );
  	++session.rspCmdIDCtr;
    ps.println( "<Target>"+session.target_dbname+"</Target><Source>"+session.source_dbname+"</Source>" );
    cb.sendCommands( sessionKey,(ISyncProtocolAdapter)this );
    ps.println( "</Sync><Final/></SyncBody></SyncML>" );
    spa.closePackage( -1, os,(IReceivingProtocolAdapter)this );
    s.p();
// Msg#3
// Send session header generated for the response message first
    s.reset();
    os = spa.sendPackage( -1, origPeer );
    ps = new PrintStream( os,true,"UTF-8" );
    ps.println( "<SyncML>" );
    session.lastMsgID++;
    ps.print( session.bos.toString( "utf-8" ) );
    session.bos = null;
    session.rspStream = ps;
    ps.println( "<Final/></SyncBody></SyncML>" );
    spa.closePackage( -1, os,(IReceivingProtocolAdapter)this );
    s.p();
    return -1;
  }

  public void abortSession( int handle ) {}

  public void suspend() {
  }
  
  public void resume() {
  }

  public int sendQuery( String sessionKey, Map metainf, String key ) throws Exception {
    SyncSession session = (SyncSession)sessions.get( sessionKey );
    if( session == null )
    	throw new SAXException( "no session object for session key "+sessionKey );
    String type = (String)metainf.get( "type" );
    int qid = queryhandles++;
    Integer qidi = new Integer( qid );
    QueryHandle qh = new QueryHandle();
    qh.cmdID = session.rspCmdIDCtr;
    qh.msgID = session.lastMsgID;
    qh.sessionKey = sessionKey;
    qh.handle = qid;
    SyncMLUtil.sendGetCommand( session.rspCmdIDCtr,type, key, session.rspStream );
  	++session.rspCmdIDCtr;
  	synchronized( queries ) {
  	  queries.put( qidi,qh );
  	}
  	return qid;
  }
  
  public void sendAddition( String sessionKey, Map metinf, String key, InputStream is ) throws Exception {
    SyncSession session = (SyncSession)sessions.get( sessionKey );
    if( session == null )
    	throw new SAXException( "no session object for session key "+sessionKey );
    String type = (String)metinf.get( "type" );
    SyncMLUtil.sendAddCommand( session.rspCmdIDCtr, key,  
  							type, is, session.rspStream );
  	++session.rspCmdIDCtr;
  }

  public void sendReplacement( String sessionKey, Map metinf, String key, InputStream is ) throws Exception {
    SyncSession session = (SyncSession)sessions.get( sessionKey );
    if( session == null )
    	throw new SAXException( "no session object for session key "+sessionKey );
    String type = (String)metinf.get( "type" );
    SyncMLUtil.sendReplaceCommand( session.rspCmdIDCtr, key,  
  							type, is, session.rspStream );
  	++session.rspCmdIDCtr;
  }

  public void sendDeletion( String sessionKey, String key ) throws Exception {
    SyncSession session = (SyncSession)sessions.get( sessionKey );
    if( session == null )
    	throw new SAXException( "no session object for session key "+sessionKey );
    SyncMLUtil.sendDeleteCommand( session.rspCmdIDCtr, key,session.rspStream ); 
  	++session.rspCmdIDCtr;
  }

// IReceivingProtocolAdapter
  public boolean receivePackage( int handle, String address, InputStream pkg ) throws Exception {
    ParserState s = new ParserState( (IReceiveCommand)this,false );
    SAXParser saxParser = pf.newSAXParser();
    XMLReader parser = saxParser.getXMLReader();
    parser.setContentHandler( s );
    InputSource src = new InputSource( pkg );
    src.setEncoding( "utf-8" );
    parser.parse( src );
    SyncSession session = s.session;
    Semaphore sem = session.notifySemaphore;
    sem.v();
    return true;
  }

// IReceiveCommand
  public void endSyncHdr( ParserState parser ) throws SAXException {
    String sessionKey = parser.sessionKey;
	if( debug ) {
	  System.out.println( "endSyncHdr: "+sessionKey );
	}
    SyncSession session = (SyncSession)sessions.get( (Object)sessionKey );
    if( session == null )
    	throw new SAXException( "no client-side session object for session key "+sessionKey );
    parser.session = session;
    session.source = parser.source;
    session.target = parser.target;
    session.lastMsgID = parser.msgID;
    session.sessionID = parser.sessionID;
    session.timestamp = System.currentTimeMillis();
    session.rspCmdIDCtr = 1;
    session.bos = new ByteArrayOutputStream();
    try {
      session.rspStream = new PrintStream( session.bos,true,"UTF-8" );
    } catch( UnsupportedEncodingException ex ) {
      throw new SAXException( ex );
    }
    try {
      SyncMLUtil.sendSyncMLHeader( session.sessionID, session.lastMsgID+1, 
  											session.target, session.source,
  											null, session.rspStream );
  	  session.rspStream.println( "<SyncBody>" );
  	  SyncMLUtil.sendStatus( session.rspCmdIDCtr, session.lastMsgID, 0, "SyncHdr",
  								session.target, session.source,
  								200, null, session.rspStream );
  	  ++session.rspCmdIDCtr;
    } catch( Exception ex ) {
      throw new SAXException( ex );
    }
  }

  public void dbAlertInPackage1( String sessionKey, int cmdID, int type, String source, String target, 
  							long last, long next ) {}
  
  public void putInPackage1( String sessionKey, int cmdID, String devinf ) {}

  public void getCommand( String sessionKey, int msgID, int cmdID, String type, String target ) {}

  public void statusCommand( String sessionKey, int msgID, int cmdID, int data, int msgRef, int cmdRef,
  				String cmd, String sourceRef, String targetRef ) {}
  
  public void resultsCommand( String sessionKey, int msgID, int cmdID, int msgRef, int cmdRef, 
      		  							String results_Source, String results_Type, 
      		  							String results_Data ) throws SAXException {
    if( !results_Source.equals( "./devinf11" ) ) {
      SyncSession session = (SyncSession)sessions.get( sessionKey );
      if( session == null )
    	throw new SAXException( "no session object for session key "+sessionKey );
      int qid = -1;
      synchronized( queries ) {
        for( Iterator i = queries.entrySet().iterator() ; i.hasNext() ; ) {
          QueryHandle qh = (QueryHandle)i.next();
          if( sessionKey.equals( qh.sessionKey ) &&
              ( cmdRef == qh.cmdID ) &&
              ( msgRef == qh.msgID ) ) {
            qid = qh.handle;
            i.remove();
            break;
          }
        }
      }
      ISyncProtocolAdapterCallBack cb = session.cb;
      HashMap m = new HashMap();
      m.put( "type",results_Type );
      ByteArrayInputStream is = new ByteArrayInputStream( results_Data.getBytes() );
      try {
        cb.syncQueryResult( sessionKey, qid, m, results_Source, is );
      } catch( Exception ex ) {
        throw new SAXException( ex );
      }
    }
  }

  public void addReplaceCommand( String sessionKey, boolean replace, int msgID, int cmdID,  
  										String source_db,String target_db,
      		  							String source, String type, 
      		  							String data ) throws SAXException {
    SyncSession session = (SyncSession)sessions.get( sessionKey );
    if( session == null )
    	throw new SAXException( "no session object for session key "+sessionKey );
    ISyncProtocolAdapterCallBack cb = session.cb;
    HashMap m = new HashMap();
    m.put( "type",type );
    ByteArrayInputStream is = new ByteArrayInputStream( data.getBytes() );
    try {
      cb.syncAddReplace( sessionKey, replace, m, source_db, target_db, source, is );
      String cmdName = replace ? "Replace" : "Add";
  	  SyncMLUtil.sendStatus( session.rspCmdIDCtr, msgID, cmdID, cmdName,
  								session.target, session.source,
  								200, null, session.rspStream );
  	  ++session.rspCmdIDCtr;
    } catch( Exception ex ) {
      ex.printStackTrace();
      throw new SAXException( ex );
    }
  }

  public void deleteCommand( String sessionKey, int msgID, int cmdID, String target_db, String target )
  							throws SAXException {
    SyncSession session = (SyncSession)sessions.get( sessionKey );
    if( session == null )
    	throw new SAXException( "no session object for session key "+sessionKey );
    ISyncProtocolAdapterCallBack cb = session.cb;
    try {
      cb.syncDelete( sessionKey, target_db, target );
  	  SyncMLUtil.sendStatus( session.rspCmdIDCtr, msgID, cmdID, "Delete",
  								session.target, session.source,
  								200, null, session.rspStream );
  	  ++session.rspCmdIDCtr;
    } catch( Exception ex ) {
      throw new SAXException( ex );
    }    
  }

  public void syncCommand( String sessionKey, int msgID, int cmdID, String source_db, String target_db )
  							throws SAXException {
    SyncSession session = (SyncSession)sessions.get( sessionKey );
    if( session == null )
    	throw new SAXException( "no session object for session key "+sessionKey );
  	SyncMLUtil.sendStatus( session.rspCmdIDCtr, msgID, cmdID, "Sync",
  								session.target, session.source,
  								200, null, session.rspStream );
  	++session.rspCmdIDCtr;
  }
  
  public void endMessage( String sessionKey, int msgID ) throws SAXException {
    SyncSession session = (SyncSession)sessions.get( sessionKey );
    if( session == null )
    	throw new SAXException( "no session object for session key "+sessionKey );
    ISyncProtocolAdapterCallBack cb = session.cb;
    if( msgID == 3 ) {
      try { 
        cb.endSession( sessionKey );
      } catch( Exception ex ) {
        throw new SAXException( ex );
      }
    }
  }
}
