package hu.uw.pallergabor.peebletest;

import android.app.Activity;
import android.content.ContentResolver;
import android.content.ContentValues;
import android.database.Cursor;
import android.net.Uri; 
import android.os.Bundle;
import android.util.Log;
import android.view.Menu;
import android.widget.Toast;

import hu.uw.pallergabor.peeble.PeebleData;
import hu.uw.pallergabor.peeble.LocDistroSyncStorageAdapter;

import java.io.ByteArrayInputStream;
import java.io.InputStream;
import java.io.IOException;
import java.io.UnsupportedEncodingException;
import java.util.HashMap;
import java.util.Map;
import java.util.Iterator;

public class PeebleTest extends Activity {
    private static final int RUN_LOCPROV_TEST = 1;
	private static final int RUN_LOCDISTROADAPT_TEST = 2;

	private static final String LOCDISTRO_CONTENT_URI_STRING =
		"content://hu.uw.pallergabor.peeble.PeebleData/LocDistro";
    private static final Uri LOCDISTRO_CONTENT_URI
		      = Uri.parse( LOCDISTRO_CONTENT_URI_STRING );
	private static final String LOCHIGHLIGHT_CONTENT_URI_STRING =
		"content://hu.uw.pallergabor.peeble.PeebleData/LocHighlight";
    private static final Uri LOCHIGHLIGHT_CONTENT_URI
		      = Uri.parse( LOCHIGHLIGHT_CONTENT_URI_STRING );
	private static final String SETTINGS_CONTENT_URI_STRING =
		"content://hu.uw.pallergabor.peeble.PeebleData/Settings";
    private static final Uri SETTINGS_CONTENT_URI
		      = Uri.parse( SETTINGS_CONTENT_URI_STRING );
	private long baseTimeStamp = 0L;

    /** Called when the activity is first created. */
    @Override
    public void onCreate(Bundle icicle)
    {
        super.onCreate(icicle);
        setContentView(R.layout.main);
    }

    @Override
    public boolean onCreateOptionsMenu(Menu menu) {
      boolean result = super.onCreateOptionsMenu(menu);
      menu.add(0, RUN_LOCPROV_TEST, R.string.runlocprovtest );
	  menu.add(0, RUN_LOCDISTROADAPT_TEST, R.string.runlocdistroadaptertest );
      return result;
    }

    @Override
    public boolean onOptionsItemSelected(Menu.Item item) {
		boolean error = false;
        switch ( item.getId() ) {
          case RUN_LOCPROV_TEST:
				error = runLocationDistributionProviderTest();
				if( error )
                	break;
				error = runLocationHighlightProviderTest();
				if( error )
					break;
				error = runSettingsProviderTest();
				break;
		  case RUN_LOCDISTROADAPT_TEST:
				error = runLocDistroAdaptTest();
				break;
		}
		if( error )
			Toast.makeText(this, "Error running test", Toast.LENGTH_SHORT).show();
        return super.onOptionsItemSelected(item);
    }

    private boolean runLocationDistributionProviderTest() {
	  if( testDeleteDatabase() )
		return true;
	  if( testInsertCoordinates() )
		return true;
	  if( testTombstone() )
		return true;
	  return false;
	}

	private boolean runLocationHighlightProviderTest() {
	  if( testDeleteHighlightDatabase() )
		return true;
	  if( testInsertHighlightDatabase() )
		return true;
	  if( testUpdateDisplayTimestamp() )
		return true;
	  if( testHighlightTombstone() )
		return true;
	  return false;
	}

	public boolean runSettingsProviderTest() {
	  if( testDeleteSettings() )
			return true;
	  if( testSettings() )
			return true;
	  return false;
	}

	public boolean runLocDistroAdaptTest() {
	  if( runLocDistroAdaptDeleteTest() )
			return true;
	  if( runLocDistroAdaptMetaInfo() )
			return true;
	  if( runLocDistroAdaptInsertTest() )
			return true;
	  return false;
	}

	private static final String LOG_TAG_DELETEDB = "PeebleTest_DeleteDatabase";

	private boolean testDeleteDatabase() {
	  Log.d( LOG_TAG_DELETEDB,"*** Starting test sequence" );
	  ContentResolver cr = getContentResolver();
// Entries in the table are really deleted and not tombstones are left
	  Uri deleteUri = Uri.withAppendedPath( LOCDISTRO_CONTENT_URI, "delete" );
	  cr.delete( deleteUri, null,null );
	  Log.d( LOG_TAG_DELETEDB,"*** Ending   test sequence" );
	  return false;
	}

	private static final String LOG_TAG_INSERTCOORD = "PeebleTest_InsertCoordinates";

	private boolean testInsertCoordinates() {
	  Log.d( LOG_TAG_INSERTCOORD,"*** Starting test sequence" );
	  ContentResolver cr = getContentResolver();

	  Log.d( LOG_TAG_INSERTCOORD,"Inserting into first coordinate bucket" );

// First coordinate: goes into bucket#1
	  ContentValues val = new ContentValues();
	  val.put( PeebleData.LocDistro.LONGITUDE_CENTER, "13.907189" );
	  val.put( PeebleData.LocDistro.LATITUDE_CENTER, "46.384814" );
	  val.put( PeebleData.LocDistro.COUNTER,"1.2" );
	  Uri resultURI = cr.insert( LOCDISTRO_CONTENT_URI,val );
	  if( resultURI == null ) {
		Log.e( LOG_TAG_INSERTCOORD, "Error: testInsertCoordinates1" );
		return true;
	  }
	  String uriStr = resultURI.toString();
	  Log.d( LOG_TAG_INSERTCOORD,"Insert result (1): "+uriStr );
	  String compareStr = LOCDISTRO_CONTENT_URI_STRING+"/location/13.907,46.385";
	  if( !uriStr.equals( compareStr  ) ) {
		Log.e( LOG_TAG_INSERTCOORD,"Error: testInsertCoordinates2, "+
							uriStr+"!="+compareStr );
		return true;
	  }
// Second coordinate: goes into bucket#1
	  val = new ContentValues();
	  val.put( PeebleData.LocDistro.LONGITUDE_CENTER, "13.907254" );
	  val.put( PeebleData.LocDistro.LATITUDE_CENTER, "46.384814" );
	  val.put( PeebleData.LocDistro.COUNTER,"1.1" );
	  resultURI = cr.insert( LOCDISTRO_CONTENT_URI,val );
	  if( resultURI == null ) {
		Log.e( LOG_TAG_INSERTCOORD, "Error: testInsertCoordinates3" );
		return true;
	  }
	  uriStr = resultURI.toString();
	  Log.d( LOG_TAG_INSERTCOORD,"Insert result (2): "+uriStr );
	  if( !uriStr.equals( LOCDISTRO_CONTENT_URI_STRING+"/location/13.907,46.385" ) ) {
		Log.e( LOG_TAG_INSERTCOORD,"Error: testInsertCoordinates4" );
		return true;
	  }

// Third coordinate: goes into bucket#2
	  Log.d( LOG_TAG_INSERTCOORD,"Inserting into the second coordinate bucket" );

	  val = new ContentValues();
	  val.put( PeebleData.LocDistro.LONGITUDE_CENTER, "13.903263" );
	  val.put( PeebleData.LocDistro.LATITUDE_CENTER, "46.381767" );
	  val.put( PeebleData.LocDistro.COUNTER,"1.2" );
	  resultURI = cr.insert( LOCDISTRO_CONTENT_URI,val );
	  if( resultURI == null ) {
		Log.e( LOG_TAG_INSERTCOORD, "Error: testInsertCoordinates5" );
		return true;
	  }
	  uriStr = resultURI.toString();
	  Log.d( LOG_TAG_INSERTCOORD,"Insert result (3): "+uriStr );
	  if( !uriStr.equals( LOCDISTRO_CONTENT_URI_STRING+"/location/13.903,46.382" ) ) {
		Log.e( LOG_TAG_INSERTCOORD,"Error: testInsertCoordinates6" );
		return true;
	  }
// Now let's see what went into the database
// bucket#1
	  String[] columns = { "counter" };
	  Uri queryUri = Uri.withAppendedPath( LOCDISTRO_CONTENT_URI, "location" );
	  queryUri = Uri.withAppendedPath( queryUri,"13.907254,46.384814" );
	  Cursor resultCursor = cr.query( queryUri, columns, null, null, null );
	  if( resultCursor == null ) {
		Log.e( LOG_TAG_INSERTCOORD,"Error: testInsertCoordinates7" );
		return true;
	  }
	  resultCursor.next();
	  int counterColumnIndex = resultCursor.getColumnIndex( "counter" );
	  String counterValueStr = resultCursor.getString( counterColumnIndex );
	  double counterValue = 0.0;
	  try {
		counterValue = Double.parseDouble( counterValueStr );
	  } catch( NumberFormatException ex ) {
	    Log.e( LOG_TAG_INSERTCOORD,"Error: testInsertCoordinates11: "+counterValueStr );
		return true;
	  }
	  Log.d( LOG_TAG_INSERTCOORD, "counterValue (1): "+counterValue );
	  if( counterValue != 2.3 ) {
		Log.e( LOG_TAG_INSERTCOORD, "Error: testInsertCoordinates8 (counterValue!=2.3 ("+counterValue+"))" );
		return true;
	  }

// bucket#2
	  queryUri = Uri.withAppendedPath( LOCDISTRO_CONTENT_URI, "location" );
	  queryUri = Uri.withAppendedPath( queryUri,"13.903263,46.381767" );
	  resultCursor = cr.query( queryUri, columns, null, null, null );
	  if( resultCursor == null ) {
		Log.e( LOG_TAG_INSERTCOORD,"Error: testInsertCoordinates9" );
		return true;
	  }
	  resultCursor.next();
	  counterColumnIndex = resultCursor.getColumnIndex( "counter" );
	  counterValueStr = resultCursor.getString( counterColumnIndex );
	  counterValue = 0.0;
	  try {
		counterValue = Double.parseDouble( counterValueStr );
	  } catch( NumberFormatException ex ) {
	    Log.e( LOG_TAG_INSERTCOORD,"Error: testInsertCoordinates12: "+counterValueStr );
		return true;
	  }
	  Log.d( LOG_TAG_INSERTCOORD, "counterValue (2): "+counterValue );
	  if( counterValue != 1.2 ) {
		Log.e( LOG_TAG_INSERTCOORD, "Error: testInsertCoordinates10 (counterValue!=1.2 ("+counterValue+"))" );
		return true;
	  }

	  Log.d( LOG_TAG_INSERTCOORD,"*** Ending   test sequence" );
	  return false;
	}

	private static final String LOG_TAG_TOMBSTONE = "PeebleTest_Tombstone";

	private boolean testTombstone() {
	  Log.d( LOG_TAG_TOMBSTONE,"*** Starting test sequence" );
	  ContentResolver cr = getContentResolver();

// this test depends on the data created by testInsertCoordinates
	  String columns[] = { "_id" };
	  Cursor c = cr.query( LOCDISTRO_CONTENT_URI, columns, null, null, null );
	  if( c == null ) {
	    Log.d( LOG_TAG_TOMBSTONE,"Tombstone (1)" );
		return true;
	  }
	  if( !c.next() ) {
	    Log.d( LOG_TAG_TOMBSTONE,"Tombstone (2)" );
		c.close();
		return true;
	  }
	  int idColumnIndex = c.getColumnIndex( "_id" );
	  long idValue = c.getLong( idColumnIndex );
	  Log.d( LOG_TAG_TOMBSTONE, "id: "+idValue );	  
// Let's delete it
	  Uri deleteUri = Uri.withAppendedPath( 
				LOCDISTRO_CONTENT_URI, Long.toString( idValue  ) );
	  cr.delete( deleteUri, null, null );
// This cannot show up in the normal query
	  c = cr.query( deleteUri, columns, null, null, null );
	  if( c != null ) {
	    if( c.next() ) {
	      idColumnIndex = c.getColumnIndex( "_id" );
		  idValue = c.getLong( idColumnIndex );
		  Log.d( LOG_TAG_TOMBSTONE, "id "+idValue+" still visible" );
		  c.close();
		  return true;
		}
		c.close();
	  }
// However, this should show up in the query with "delete" URI 
	  deleteUri = Uri.withAppendedPath( 
						LOCDISTRO_CONTENT_URI,
						"delete" );
	  deleteUri = Uri.withAppendedPath( deleteUri, Long.toString( idValue ) );
	  c = cr.query( deleteUri, columns, null, null, null );
	  if( c == null ) {
		Log.d( LOG_TAG_TOMBSTONE, "Tombstone (3)" );
		return true;
	  }
	  if( !c.next() ) {
		Log.d( LOG_TAG_TOMBSTONE, "Tombstone (4)" );
		c.close();
		return true;
	  }
	  int idColumnIndex2 = c.getColumnIndex( "_id" );
	  long idValue2 = c.getLong( idColumnIndex2 );
	  if( idValue != idValue2 ) {
		Log.d( LOG_TAG_TOMBSTONE, "idValue ("+idValue+") != idValue2("+idValue2+")" );
		return true;
	  }
	  Log.d( LOG_TAG_TOMBSTONE,"*** Ending   test sequence" );
	  return false;
	}

	private static final String LOG_TAG_DELETEHIGHLIGHTDB = "PeebleTest_DeleteHighlightDatabase";

	private boolean testDeleteHighlightDatabase() {
	  Log.d( LOG_TAG_DELETEHIGHLIGHTDB,"*** Starting test sequence" );
	  ContentResolver cr = getContentResolver();
// Entries in the table are really deleted and not tombstones are left
	  Uri deleteUri = Uri.withAppendedPath( LOCHIGHLIGHT_CONTENT_URI, "delete" );
	  cr.delete( deleteUri, null,null );
	  Log.d( LOG_TAG_DELETEHIGHLIGHTDB,"*** Ending   test sequence" );
	  return false;
	}


	private static final String LOG_TAG_INSERTHIGHLIGHT = "PeebleTest_InsertHighlightDatabase";

	private boolean testInsertHighlightDatabase() {
	  Log.d( LOG_TAG_INSERTHIGHLIGHT,"*** Starting test sequence" );
	  ContentResolver cr = getContentResolver();

	  Log.d( LOG_TAG_INSERTHIGHLIGHT,"Inserting into first coordinate bucket" );

	  ContentValues val = new ContentValues();
	  val.put( PeebleData.LocHighlight.EXACT_LONGITUDE, "13.907189" );
	  val.put( PeebleData.LocHighlight.EXACT_LATITUDE, "46.384814" );
	  val.put( PeebleData.LocHighlight.MATCHING_TERMS, "wine shop" );
	  val.put( PeebleData.LocHighlight.ADDRESS, "120 Raspberry road, Fulton, XC87 1QD" );
	  val.put( PeebleData.LocHighlight.DESCRIPTION, "Wonderful wine shop" );
	  val.put( PeebleData.LocHighlight.DISPLAY_TIMESTAMP, new Long( 0L ) );
	  Uri resultURI = cr.insert( LOCHIGHLIGHT_CONTENT_URI,val );
	  if( resultURI == null ) {
		Log.e( LOG_TAG_INSERTHIGHLIGHT, "Error: testInsertHighlight1" );
		return true;
	  }
	  String uriStr = resultURI.toString();
	  Log.d( LOG_TAG_INSERTHIGHLIGHT,"Insert result (1): "+uriStr );
	  String compareStr = LOCHIGHLIGHT_CONTENT_URI_STRING+"/location/13.907,46.385";
	  if( !uriStr.equals( compareStr  ) ) {
		Log.e( LOG_TAG_INSERTHIGHLIGHT,"Error: testInsertHighlight2, "+
							uriStr+"!="+compareStr );
		return true;
	  }

	  val.clear();
	  val.put( PeebleData.LocHighlight.EXACT_LONGITUDE, "13.907254" );
	  val.put( PeebleData.LocHighlight.EXACT_LATITUDE, "46.384814" );
	  val.put( PeebleData.LocHighlight.MATCHING_TERMS, "Free WiFi Access" );
	  val.put( PeebleData.LocHighlight.ADDRESS, "100 Raspberry road, Fulton, XC87 1QD" );
	  val.put( PeebleData.LocHighlight.DESCRIPTION, "Open WiFi at the square" );
	  val.put( PeebleData.LocHighlight.DISPLAY_TIMESTAMP, new Long( 0L ) );
	  resultURI = cr.insert( LOCHIGHLIGHT_CONTENT_URI,val );
	  if( resultURI == null ) {
		Log.e( LOG_TAG_INSERTHIGHLIGHT, "Error: testInsertHighlight3" );
		return true;
	  }
	  uriStr = resultURI.toString();
	  Log.d( LOG_TAG_INSERTHIGHLIGHT,"Insert result (2): "+uriStr );
	  compareStr = LOCHIGHLIGHT_CONTENT_URI_STRING+"/location/13.907,46.385";
	  if( !uriStr.equals( compareStr  ) ) {
		Log.e( LOG_TAG_INSERTHIGHLIGHT,"Error: testInsertHighlight4, "+
							uriStr+"!="+compareStr );
		return true;
	  }

// bucket#2
	  Log.d( LOG_TAG_INSERTHIGHLIGHT,"Inserting into the second coordinate bucket" );

	  val.clear();
	  val.put( PeebleData.LocHighlight.EXACT_LONGITUDE, "13.903263" );
	  val.put( PeebleData.LocHighlight.EXACT_LATITUDE, "46.381767" );
	  val.put( PeebleData.LocHighlight.MATCHING_TERMS, "post office" );
	  val.put( PeebleData.LocHighlight.ADDRESS, "12 Strawberry road, Fulton, XC85 1QA" );
	  val.put( PeebleData.LocHighlight.DESCRIPTION, "Post office and supermarket" );
	  val.put( PeebleData.LocHighlight.DISPLAY_TIMESTAMP, new Long( 0L ) );
	  resultURI = cr.insert( LOCHIGHLIGHT_CONTENT_URI,val );
	  if( resultURI == null ) {
		Log.e( LOG_TAG_INSERTHIGHLIGHT, "Error: testInsertHighlight5" );
		return true;
	  }
	  uriStr = resultURI.toString();
	  Log.d( LOG_TAG_INSERTHIGHLIGHT,"Insert result (2): "+uriStr );
	  compareStr = LOCHIGHLIGHT_CONTENT_URI_STRING+"/location/13.903,46.382";
	  if( !uriStr.equals( compareStr  ) ) {
		Log.e( LOG_TAG_INSERTHIGHLIGHT,"Error: testInsertHighlight6, "+
							uriStr+"!="+compareStr );
		return true;
	  }

// Now let's see what went into the database
// bucket#1
	  String[] columns = { "exact_longitude",
							"exact_latitude",
							"address",
							"description",
							"matching_terms" };
	  Uri queryUri = Uri.withAppendedPath( LOCHIGHLIGHT_CONTENT_URI, "location" );
	  queryUri = Uri.withAppendedPath( queryUri,"13.907151,46.384924" );
	  Cursor resultCursor = cr.query( queryUri, columns, null, null, null );
	  if( resultCursor == null ) {
		Log.e( LOG_TAG_INSERTHIGHLIGHT,"Error: testInsertHighlight7" );
		return true;
	  }
	  if( !resultCursor.next() ) {
		Log.e( LOG_TAG_INSERTHIGHLIGHT,"Error: testInsertHighlight8" );
		return true;
	  }
	  int longColumnIndex = resultCursor.getColumnIndex( "exact_longitude" );
	  int latColumnIndex = resultCursor.getColumnIndex( "exact_latitude" );
	  int addressColumnIndex = resultCursor.getColumnIndex( "address" );
	  int descriptionColumnIndex = resultCursor.getColumnIndex( "description" );
	  int matchingTermsColumnIndex = resultCursor.getColumnIndex( "matching_terms" );

	  String longValue = resultCursor.getString( longColumnIndex );
	  String latValue = resultCursor.getString( latColumnIndex );
	  String addressVal = resultCursor.getString( addressColumnIndex );
	  String descriptionVal = resultCursor.getString( descriptionColumnIndex );
	  String matchingTermsVal = resultCursor.getString( matchingTermsColumnIndex );
	  Log.d( LOG_TAG_INSERTHIGHLIGHT, "bucket#1: exact_longitude: "+longValue+
								" ,exact_latitude: "+latValue+
								" ,address: "+addressVal+
								" ,description: "+descriptionVal+
								" ,matching terms: "+matchingTermsVal );
	  if( checkTestedValue( longValue, "13.907254" , 
								LOG_TAG_INSERTHIGHLIGHT, "testHighlight11_1" ) )
		return true;
	  if( checkTestedValue( latValue, "46.384814" , 
								LOG_TAG_INSERTHIGHLIGHT, "testHighlight11_2" ) )
		return true;
	  if( checkTestedValue( addressVal, "100 Raspberry road, Fulton, XC87 1QD" , 
								LOG_TAG_INSERTHIGHLIGHT, "testHighlight11_3" ) )
		return true;
	  if( checkTestedValue( descriptionVal, "Open WiFi at the square" , 
								LOG_TAG_INSERTHIGHLIGHT, "testHighlight11_4" ) )
		return true;
	  if( checkTestedValue( matchingTermsVal, "Free WiFi Access" , 
								LOG_TAG_INSERTHIGHLIGHT, "testHighlight11_5" ) )
		return true;
	  if( !resultCursor.next() ) {
		Log.e( LOG_TAG_INSERTHIGHLIGHT,"Error: testInsertHighlight8" );
		return true;
	  }
	  longValue = resultCursor.getString( longColumnIndex );
	  latValue = resultCursor.getString( latColumnIndex );
	  addressVal = resultCursor.getString( addressColumnIndex );
	  descriptionVal = resultCursor.getString( descriptionColumnIndex );
	  matchingTermsVal = resultCursor.getString( matchingTermsColumnIndex );
	  Log.d( LOG_TAG_INSERTHIGHLIGHT, "bucket#1: exact_longitude: "+longValue+
								" ,exact_latitude: "+latValue+
								" ,address: "+addressVal+
								" ,description: "+descriptionVal+
								" ,matching terms: "+matchingTermsVal );
	  if( checkTestedValue( longValue, "13.907189" , 
								LOG_TAG_INSERTHIGHLIGHT, "testHighlight12_1" ) )
		return true;
	  if( checkTestedValue( latValue, "46.384814" , 
								LOG_TAG_INSERTHIGHLIGHT, "testHighlight12_2" ) )
		return true;
	  if( checkTestedValue( addressVal, "120 Raspberry road, Fulton, XC87 1QD" , 
								LOG_TAG_INSERTHIGHLIGHT, "testHighlight12_3" ) )
		return true;
	  if( checkTestedValue( descriptionVal, "Wonderful wine shop" , 
								LOG_TAG_INSERTHIGHLIGHT, "testHighlight12_4" ) )
		return true;
	  if( checkTestedValue( matchingTermsVal, "wine shop" , 
								LOG_TAG_INSERTHIGHLIGHT, "testHighlight12_5" ) )
		return true;
	  resultCursor.close();

// bucket#2
	  queryUri = Uri.withAppendedPath( LOCHIGHLIGHT_CONTENT_URI, "location" );
	  queryUri = Uri.withAppendedPath( queryUri,"13.903152,46.381857" );
	  resultCursor = cr.query( queryUri, columns, null, null, null );
	  if( resultCursor == null ) {
		Log.e( LOG_TAG_INSERTHIGHLIGHT,"Error: testInsertHighlight9" );
		return true;
	  }
	  if( !resultCursor.next() ) {
		Log.e( LOG_TAG_INSERTHIGHLIGHT,"Error: testInsertHighlight10" );
		return true;
	  }
	  longValue = resultCursor.getString( longColumnIndex );
	  latValue = resultCursor.getString( latColumnIndex );
	  addressVal = resultCursor.getString( addressColumnIndex );
	  descriptionVal = resultCursor.getString( descriptionColumnIndex );
	  matchingTermsVal = resultCursor.getString( matchingTermsColumnIndex );
	  Log.d( LOG_TAG_INSERTHIGHLIGHT, "bucket#2: exact_longitude: "+longValue+
								" ,exact_latitude: "+latValue+
								" ,address: "+addressVal+
								" ,description: "+descriptionVal+
								" ,matching terms: "+matchingTermsVal );
	  if( checkTestedValue( longValue, "13.903263" , 
								LOG_TAG_INSERTHIGHLIGHT, "testHighlight13_1" ) )
		return true;
	  if( checkTestedValue( latValue, "46.381767" , 
								LOG_TAG_INSERTHIGHLIGHT, "testHighlight13_2" ) )
		return true;
	  if( checkTestedValue( addressVal, "12 Strawberry road, Fulton, XC85 1QA" , 
								LOG_TAG_INSERTHIGHLIGHT, "testHighlight13_3" ) )
		return true;
	  if( checkTestedValue( descriptionVal, "Post office and supermarket" , 
								LOG_TAG_INSERTHIGHLIGHT, "testHighlight13_4" ) )
		return true;
	  if( checkTestedValue( matchingTermsVal, "post office" , 
								LOG_TAG_INSERTHIGHLIGHT, "testHighlight13_5" ) )
		return true;
	  resultCursor.close();
	  Log.d( LOG_TAG_INSERTHIGHLIGHT,"*** Ending   test sequence" );
	  return false;	  
	}

	private static final String LOG_TAG_HIGHLIGHTTOMBSTONE = "PeebleTest_HighlightTombstone";

	private boolean testHighlightTombstone() {
	  Log.d( LOG_TAG_HIGHLIGHTTOMBSTONE,"*** Starting test sequence" );
	  ContentResolver cr = getContentResolver();

// this test depends on the data created by testInsertCoordinates
	  String columns[] = { "_id" };
	  Cursor c = cr.query( LOCHIGHLIGHT_CONTENT_URI, columns, null, null, null );
	  if( c == null ) {
	    Log.d( LOG_TAG_HIGHLIGHTTOMBSTONE,"Tombstone (1)" );
		return true;
	  }
	  if( !c.next() ) {
	    Log.d( LOG_TAG_HIGHLIGHTTOMBSTONE,"Tombstone (2)" );
		c.close();
		return true;
	  }
	  int idColumnIndex = c.getColumnIndex( "_id" );
	  long idValue = c.getLong( idColumnIndex );
	  Log.d( LOG_TAG_HIGHLIGHTTOMBSTONE, "id: "+idValue );	  
// Let's delete it
	  Uri deleteUri = Uri.withAppendedPath( 
				LOCHIGHLIGHT_CONTENT_URI, Long.toString( idValue  ) );
	  cr.delete( deleteUri, null, null );
// This cannot show up in the normal query
	  c = cr.query( deleteUri, columns, null, null, null );
	  if( c != null ) {
	    if( c.next() ) {
	      idColumnIndex = c.getColumnIndex( "_id" );
		  idValue = c.getLong( idColumnIndex );
		  Log.d( LOG_TAG_HIGHLIGHTTOMBSTONE, "id "+idValue+" still visible" );
		  c.close();
		  return true;
		}
		c.close();
	  }
// However, this should show up in the query with "delete" URI 
	  deleteUri = Uri.withAppendedPath( 
						LOCHIGHLIGHT_CONTENT_URI,
						"delete" );
	  deleteUri = Uri.withAppendedPath( deleteUri, Long.toString( idValue ) );
	  c = cr.query( deleteUri, columns, null, null, null );
	  if( c == null ) {
		Log.d( LOG_TAG_HIGHLIGHTTOMBSTONE, "Tombstone (3)" );
		return true;
	  }
	  if( !c.next() ) {
		Log.d( LOG_TAG_HIGHLIGHTTOMBSTONE, "Tombstone (4)" );
		c.close();
		return true;
	  }
	  int idColumnIndex2 = c.getColumnIndex( "_id" );
	  long idValue2 = c.getLong( idColumnIndex2 );
	  if( idValue != idValue2 ) {
		Log.d( LOG_TAG_HIGHLIGHTTOMBSTONE, "idValue ("+idValue+") != idValue2("+idValue2+")" );
		return true;
	  }
	  Log.d( LOG_TAG_HIGHLIGHTTOMBSTONE,"*** Ending   test sequence" );
	  return false;
	}

	private static final String LOG_TAG_UPDATEDTSTAMP = "PeebleTest_UpdateDisplayTimestamp";

// This test depends on the database entries created by testInsertHighlightDatabase
	private boolean testUpdateDisplayTimestamp() {
	  Log.d( LOG_TAG_UPDATEDTSTAMP,"*** Starting test sequence" );
	  ContentResolver cr = getContentResolver();
// Go for bucket#2
	  Uri queryUri = Uri.withAppendedPath( LOCHIGHLIGHT_CONTENT_URI, "location" );
	  queryUri = Uri.withAppendedPath( queryUri,"13.903152,46.381857" );
	  String idColumn[] = { "_id" };
	  Cursor resultCursor = cr.query( queryUri, idColumn, null, null, null );
	  if( resultCursor == null ) {
		Log.e( LOG_TAG_UPDATEDTSTAMP,"Error: testUpdateDisplayTimestamp1" );
		return true;
	  }
	  if( !resultCursor.next() ) {
		Log.e( LOG_TAG_UPDATEDTSTAMP,"Error: testUpdateDisplayTimestamp2" );
		return true;
	  }
	  int idColumnIndex = resultCursor.getColumnIndex( "_id" );
	  String idValue = resultCursor.getString( idColumnIndex );
	  Log.d( LOG_TAG_UPDATEDTSTAMP, "ID found: "+idValue );
	  resultCursor.close();

	  long baseTimeStamp = System.currentTimeMillis();
	  Uri updateUri = Uri.withAppendedPath( LOCHIGHLIGHT_CONTENT_URI, 
			"displaytimestamp" );
	  updateUri = Uri.withAppendedPath( updateUri, idValue );
	  Log.d( LOG_TAG_UPDATEDTSTAMP,"updateURI: "+updateUri.toString() );

	  cr.update( updateUri, new ContentValues(), null, null );

	  queryUri = Uri.withAppendedPath( LOCHIGHLIGHT_CONTENT_URI, idValue );
	  String tsColumn[] = { "display_timestamp" };
	  resultCursor = cr.query( queryUri, tsColumn, null, null, null );
	  if( !resultCursor.next() ) {
		Log.e( LOG_TAG_UPDATEDTSTAMP,"Error: testUpdateDisplayTimestamp3" );
		return true;
	  }
	  int dtStampColumnIndex = resultCursor.getColumnIndex( "display_timestamp" );
	  long dtStamp = resultCursor.getLong( dtStampColumnIndex );
	  Log.d( LOG_TAG_UPDATEDTSTAMP, "Display timestamp "+
						dtStamp+
						" base: "+
						baseTimeStamp );
	  if( ( dtStamp < baseTimeStamp ) || 
		  ( dtStamp > ( baseTimeStamp + 10000L ) ) ) {
		Log.e( LOG_TAG_UPDATEDTSTAMP, "Timestamp out of bounds, base: "+
						baseTimeStamp+" ,display timestamp: "+dtStamp );
		return true;
	  }
	  Log.d( LOG_TAG_UPDATEDTSTAMP,"*** Ending   test sequence" );
	  return false;
	}


	private static final String LOG_TAG_DELETE_SETTINGS = "PeebleTest_DeleteSettings";

	private boolean testDeleteSettings() {
	  Log.d( LOG_TAG_DELETE_SETTINGS,"*** Starting test sequence" );
	  ContentResolver cr = getContentResolver();
	  cr.delete( SETTINGS_CONTENT_URI, null,null );
	  Log.d( LOG_TAG_DELETE_SETTINGS,"*** Ending   test sequence" );
	  return false;
	}

	private static final String LOG_TAG_SETTINGS = "PeebleTest_Settings";

	private boolean testSettings() {
	  Log.d( LOG_TAG_SETTINGS,"*** Starting test sequence" );
	  ContentResolver cr = getContentResolver();

// Insert name-value pair
	  Log.d( LOG_TAG_SETTINGS,"Inserting a name-value pair" );

	  ContentValues val = new ContentValues();
	  val.put( PeebleData.Settings.NAME, "testname" );
	  val.put( PeebleData.Settings.VALUE, "testvalue" );
	  Uri resultURI = cr.insert( SETTINGS_CONTENT_URI,val );
	  if( resultURI == null ) {
		Log.e( LOG_TAG_SETTINGS, "Error: testSettings1" );
		return true;
	  }
	  String uriStr = resultURI.toString();
	  Log.d( LOG_TAG_SETTINGS,"Insert result: "+uriStr );
	  String compareStr = SETTINGS_CONTENT_URI_STRING+"/1";
	  if( !uriStr.equals( compareStr  ) ) {
		Log.e( LOG_TAG_SETTINGS,"Error: testSettings2, "+
							uriStr+"!="+compareStr );
		return true;
	  }

// Read it back
	  String settingsColumn[] = { "_id", "name", "value" };
	  Cursor resultCursor = cr.query( PeebleData.Settings.CONTENT_URI, 
				settingsColumn, 
				null, 
				null, 
				null );
	  if( !resultCursor.next() ) {
		Log.e( LOG_TAG_SETTINGS,"Error: testSettings3" );
		return true;
	  }
	  int idColumnIndex = resultCursor.getColumnIndex( "_id" );
	  int nameColumnIndex = resultCursor.getColumnIndex( "name" );
	  int valueColumnIndex = resultCursor.getColumnIndex( "value" );
	  String idValue = resultCursor.getString( idColumnIndex );
	  String nameValue = resultCursor.getString( nameColumnIndex );
	  String valueValue = resultCursor.getString( valueColumnIndex );
	  resultCursor.close();
	  Log.d( LOG_TAG_SETTINGS,"_id: "+idValue+
							",name: "+nameValue+
							",value: "+valueValue );
	  if( checkTestedValue( idValue,"1",LOG_TAG_SETTINGS, "testSettings4" ) )
		return true;
	  if( checkTestedValue( nameValue,"testname",LOG_TAG_SETTINGS, "testSettings5" ) )
		return true;
	  if( checkTestedValue( valueValue,"testvalue",LOG_TAG_SETTINGS, "testSettings6" ) )
		return true;
	  val = new ContentValues();
	  val.put( "value","testvalue2" );
	  cr.update( PeebleData.Settings.CONTENT_URI,val,"name='testname'",null );
	  resultCursor = cr.query( PeebleData.Settings.CONTENT_URI, 
				settingsColumn, 
				null, 
				null, 
				null );
	  if( !resultCursor.next() ) {
		Log.e( LOG_TAG_SETTINGS,"Error: testSettings7" );
		return true;
	  }

	  idValue = resultCursor.getString( idColumnIndex );
	  nameValue = resultCursor.getString( nameColumnIndex );
	  valueValue = resultCursor.getString( valueColumnIndex );
	  resultCursor.close();
	  Log.d( LOG_TAG_SETTINGS,"_id: "+idValue+
							",name: "+nameValue+
							",value: "+valueValue );
	  if( checkTestedValue( idValue,"1",LOG_TAG_SETTINGS, "testSettings8" ) )
		return true;
	  if( checkTestedValue( nameValue,"testname",LOG_TAG_SETTINGS, "testSettings9" ) )
		return true;
	  if( checkTestedValue( valueValue,"testvalue2",LOG_TAG_SETTINGS, "testSettings10" ) )
		return true;

	  Log.d( LOG_TAG_SETTINGS,"*** Ending   test sequence" );
	  return false;
	}

	private boolean checkTestedValue( String value, String expectedValue, 
								String mainTag, String subTag ) {
	  if( !expectedValue.equals( value ) ) {
		Log.e( mainTag, subTag+" -> got: "+value+" ,expected: "+expectedValue );
		return true;
	  }
	  return false;
	} 

	private static final String LOG_TAG_LOCDISTROADAPTDELETE = 
					"PeebleTest_LocDistroAdaptDelete";

	public boolean runLocDistroAdaptDeleteTest() {
	  baseTimeStamp = System.currentTimeMillis();
	  Log.d( LOG_TAG_LOCDISTROADAPTDELETE,"*** Starting test sequence" );

	  ContentResolver cr = getContentResolver();
// Metainfo table emptied
	  cr.delete( PeebleData.LocDistro.SYNCMETAINFO_CONTENT_URI, null,null );

// Entries in the table are really deleted and no tombstones are left
	  Uri deleteUri = Uri.withAppendedPath( PeebleData.LocDistro.CONTENT_URI, "delete" );
	  cr.delete( deleteUri, null,null );
	  
	  Log.d( LOG_TAG_LOCDISTROADAPTDELETE,"*** Ending   test sequence" );
	  return false;
	}

	private static final String LOG_TAG_LOCDISTROADAPTMETAINFO = "PeebleTest_LocDistroAdaptMetaInfo";

	public boolean runLocDistroAdaptMetaInfo() {
	  Log.d( LOG_TAG_LOCDISTROADAPTMETAINFO,"*** Starting test sequence" );
	  ContentResolver cr = getContentResolver();
	  LocDistroSyncStorageAdapter ls = new LocDistroSyncStorageAdapter( cr );
	  HashMap m = new HashMap();
	  m.put( "distro_last",Long.toString( baseTimeStamp ) );
	  ls.setDatabaseMetaInfo( (Map)m );
	  Map m2 = ls.getDatabaseMetaInfo();
	  String dstampString = (String)m2.get( "distro_last" );
	  long dstamp = 0;
	  try {
	    dstamp = Long.parseLong( dstampString );
	  } catch( NumberFormatException ex ) {
		Log.e( LOG_TAG_LOCDISTROADAPTMETAINFO, "dstampString cannot be parsed: "+
					dstampString );
		return true;
	  }
	  if( dstamp != baseTimeStamp ) {
		Log.e( LOG_TAG_LOCDISTROADAPTMETAINFO, "timestamp mismatch: baseTimeStamp: "+
					baseTimeStamp+", dstamp: "+dstamp );
		return true;
	  }
	  Log.d( LOG_TAG_LOCDISTROADAPTMETAINFO,"*** Ending   test sequence" );	  
	  return false;
	}

	private static final String LOG_TAG_LOCDISTROADAPTINSERT = "PeebleTest_LocDistroAdaptInsert";

	public boolean runLocDistroAdaptInsertTest() {
	  Log.d( LOG_TAG_LOCDISTROADAPTINSERT,"*** Starting test sequence" );
	  ContentResolver cr = getContentResolver();
	  LocDistroSyncStorageAdapter ls = new LocDistroSyncStorageAdapter( cr );
	  String testData1 = "14.903,47.381,1.5";
	  String testData2 = "15.103,48.901,0.1";
      ByteArrayInputStream bis1 = null;
      ByteArrayInputStream bis2 = null;
	  try {
	    bis1 = new ByteArrayInputStream( testData1.getBytes( "UTF-8") );
	    bis2 = new ByteArrayInputStream( testData2.getBytes( "UTF-8") );
	  } catch( UnsupportedEncodingException ex ) {
		Log.e( LOG_TAG_LOCDISTROADAPTINSERT,"UnsupportedEncodingException"  );
		return true;
	  }
	  ls.setEntryData( "","",bis1 );
	  HashMap m = new HashMap();
	  Log.d( LOG_TAG_LOCDISTROADAPTINSERT, "baseTimeStamp: "+baseTimeStamp );
	  long pastTs = baseTimeStamp - 1000L;
	  m.put( "created",Long.toString( pastTs ) );
	  m.put( "timestamp", Long.toString( pastTs ) );
	  ls.setEntryData( "",(Map)m,bis2 );
	  HashMap queryMap = new HashMap();
	  queryMap.put( "timestampgeq",new Long( baseTimeStamp ) );
	  Iterator it = ls.selectKeys( queryMap );
	  if( it == null ) {
		Log.e( LOG_TAG_LOCDISTROADAPTINSERT, "it==null" );
		return true;
	  }
	  int ctr = 0;
	  for( ; it.hasNext() ; ) {
	    String key = (String)it.next();
		InputStream is = ls.getEntryData( key );
		StringBuffer b = new StringBuffer();
    	int ch = 0;
		try {
      	  do {
            ch = is.read();
        	if( ch >= 0 )
          			b.append( (char)ch );
      	  } while( ch >= 0 );
		} catch( IOException ex ) {
		  Log.e( LOG_TAG_LOCDISTROADAPTINSERT, "IOException" );
	  	  return true;
		}
		String s = new String( b );
		Log.d( LOG_TAG_LOCDISTROADAPTINSERT, "key: "+key+", value: "+s );
		if( !testData1.equals( s ) ) {
		  Log.e( LOG_TAG_LOCDISTROADAPTINSERT, "invalid value found: "+s );
		  return true;
		}
		++ctr;
	  }
	  if( ctr > 1 ) {
		Log.e( LOG_TAG_LOCDISTROADAPTINSERT, "ctr>1: "+ctr );
		return true;
	  }
	  Log.d( LOG_TAG_LOCDISTROADAPTINSERT,"*** Ending   test sequence" );	  
	  return false;
	}
}
