import org.xml.sax.XMLReader;
import org.xml.sax.InputSource;
import org.xml.sax.Attributes;
import org.xml.sax.SAXException;
import org.xml.sax.helpers.XMLReaderFactory;
import org.xml.sax.helpers.DefaultHandler;
import java.util.List;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.StringTokenizer;
import java.util.NoSuchElementException;
import java.util.GregorianCalendar;
import java.io.FileReader;
import java.io.PrintWriter;
import java.io.FileWriter;
import java.io.IOException;
import java.io.ByteArrayInputStream;

public class gfxtotrack extends DefaultHandler {
  static final int IN_BODY = 0;
  static final int IN_GPX = 1;
  static final int IN_TRK = 2;
  static final int IN_NAME = 3;
  static final int IN_TRKSEG = 4;
  static final int IN_TRKPT = 5;
  static final int IN_ELE = 6;
  static final int IN_TIME = 7;
  int parseState;

  LocationStorageEntry actualEntry = null;
  String collectString = null;
  long timeBase = -1L;
  PrintWriter outf = null;
  String baseName = null;
  long speedFactor = 1L;

  public void setOutput( String baseName ) {
	this.baseName = baseName;
  }

  public void setSpeedFactor( long speedFactor ) {
	this.speedFactor = speedFactor;
  }

/**
  * Reads the location file.
  * @param locationFile Name of the location file.
  * @throw SAXException if there was a parse error.
  * Document structure: see locations.xml
  */
  public void readLocationFile( String locationFile ) throws SAXException {
    XMLReader xr = XMLReaderFactory.createXMLReader();
    xr.setContentHandler( this );
    xr.setErrorHandler( this );
	try {
	 parseState = IN_BODY;
     FileReader r = new FileReader( locationFile );
     xr.parse( new InputSource( r ) );
	} catch( IOException ex ) {
	  throw new SAXException( ex );
	}
	if( outf != null )
		outf.close();
	if( actualEntry != null ) {
	  long tracklenInSec = actualEntry.time/1000L;
	  System.out.println( "Total track length: "+
						tracklenInSec+
						" seconds ("+
						((float)tracklenInSec/60.0f)+" minutes)" );
	}
  }

  public void startElement (String uri, String name,
                              String qName, Attributes atts) throws SAXException {
	if( qName.equals( "gpx" ) ) {
	  if( parseState == IN_BODY )
		parseState = IN_GPX;
	  else
		throw new SAXException( "Misplaced <gpx>" );
	} else
	if( qName.equals( "trk" ) ) {
	  if( parseState == IN_GPX ) {
	    parseState = IN_TRK;
	  } else
		throw new SAXException( "Misplaced <trk>" );
	} else
	if( qName.equals( "name" ) ) {
	  if( parseState == IN_TRK ) {
		parseState = IN_NAME;
	  } else
		throw new SAXException( "Misplaced <name>" );
	} else
	if( qName.equals( "trkseg" ) ) {
	  if( parseState == IN_TRK ) {
		parseState = IN_TRKSEG;
	  } else
		throw new SAXException( "Misplaced <trkseg>" );
	} else
	if( qName.equals( "trkpt" ) ) {
	  if( parseState == IN_TRKSEG ) {
		actualEntry = new LocationStorageEntry();
		String longitude = atts.getValue( "lon" );
		if( longitude == null )
			throw new SAXException( "'lon' attribute is missing on <trkpt>" );
		String latitude = atts.getValue( "lat" );
		if( latitude == null )
			throw new SAXException( "'lat' attribute is missing on <trkpt>" );
		actualEntry.longitude = longitude;
		actualEntry.latitude = latitude;
		parseState = IN_TRKPT;
	  } else
		throw new SAXException( "Misplaced <trkpt>" );
	} else
	if( qName.equals( "ele" ) ) {
	  if( parseState == IN_TRKPT ) {
		parseState = IN_ELE;
	  } else
		throw new SAXException( "Misplaced <ele>" );
	} else
	if( qName.equals( "time" ) ) {
	  if( parseState == IN_TRKPT ) {
		collectString = "";
		parseState = IN_TIME;
	  } else
		throw new SAXException( "Misplaced <time>" );
	} else
		throw new SAXException( "Unknown element: "+qName );
  }

  public void endElement (String uri, String name, String qName) throws SAXException {
    if ( qName.equals( "gpx" ) )
		parseState = IN_BODY;
	else
	if( qName.equals( "trk" ) ) {
	  parseState = IN_GPX;
	} else
	if( qName.equals( "name" ) ) {
	  parseState = IN_TRK;
	} else
	if( qName.equals( "trkseg" ) ) {
	  parseState = IN_TRK;
	} else
	if( qName.equals( "trkpt" ) ) {
	  if( !actualEntry.validate() )
			throw new SAXException( "invalid <trkpt> content: "+actualEntry );
	  try {
		writeLocationStorageEntry( actualEntry );
	  } catch( IOException ex ) {
		throw new SAXException( ex );
	  }
	  System.out.println( "trkpt: "+actualEntry );
	  parseState = IN_TRKSEG;
	} else
	if( qName.equals( "ele" ) ) {
	  parseState = IN_TRKPT;
	} else
	if( qName.equals( "time" ) ) {
	  StringTokenizer st = new StringTokenizer( collectString );
	  String s = null;
	  try {
		s = st.nextToken( "-" );
		int dayOfMonth = Integer.parseInt( s  );
		s = st.nextToken( "-" );
		int month = Integer.parseInt( s );
		s = st.nextToken( "T" );
		int year = Integer.parseInt( s.substring( 1 ) );
		s = st.nextToken( ":" );
		int hour = Integer.parseInt( s.substring( 1 ) );
		s = st.nextToken( ":" );
		int minutes = Integer.parseInt( s );
		s = st.nextToken( "Z" );
		int seconds = Integer.parseInt( s.substring( 1 )  );
//		System.out.println( year+"/"+month+"/"+dayOfMonth+" "+hour+":"+minutes+":"+seconds );
	    GregorianCalendar gc = new GregorianCalendar( year,month,dayOfMonth,
				hour,minutes,seconds );
		actualEntry.time = gc.getTimeInMillis();
	  } catch( NoSuchElementException ex ) {
		throw new SAXException( "Malformed <time>: "+collectString );
	  } catch( NumberFormatException ex ) {
		throw new SAXException( "Malformed number in <time> element: "+collectString+" ("+s+")" );
	  }
	  parseState = IN_TRKPT;
    }
  }

  public void characters( char ch[], int start, int length) {
	if( parseState == IN_TIME ) {
	  String s = new String( ch, start, length );
	  collectString += s;
	}
  }

  public static void main( String args[] ) {
	 String inpFile = null;
	 String provName = null;
     int speedFactor = 1;
	 for( int i = 0 ; i < args.length ; ++i ) {
	   String arg = args[i];
	   if( arg.startsWith( "-s" ) ) {
		 try {
		   speedFactor = Integer.parseInt( arg.substring( 2 ) );
		 } catch( NumberFormatException ex ) {
		   usage();
		   return;
	     }
	   } else {
	     if( inpFile == null )
			inpFile = arg;
         else
		 if( provName == null )
			provName = arg;
		 else {
		   usage();
		   return;
		 }
	   }
	 }
	 if( inpFile == null || provName == null ) {
	   usage();
	   return;
	 }
	gfxtotrack st = new gfxtotrack();
	try {
	  st.setOutput( provName );
	  st.setSpeedFactor( speedFactor );
	  st.readLocationFile( inpFile );
	} catch( Exception ex ) {
	  ex.printStackTrace();
	}
  }

  private void writeLocationStorageEntry( LocationStorageEntry lse ) throws IOException {
	if( timeBase < 0 ) { // first track point 
	  timeBase = lse.time;
	  String outFileName = baseName +"_track";
	  String locFileName = baseName +"_location";
	  FileWriter f = new FileWriter( outFileName );	   
	  outf = new PrintWriter( f );
	  FileWriter lf = new FileWriter( locFileName );
	  PrintWriter outlf = new PrintWriter( lf );
	  writeLocation( outlf, baseName, lse );
	  outlf.close();
	}
	lse.time -= timeBase;
	lse.time /= speedFactor;
	writeTrackLine( outf,lse );
  }

  private void writeTrackLine( PrintWriter outf, LocationStorageEntry lse ) throws IOException {
    outf.println( lse.time+" "+lse.longitude+" "+lse.latitude+" 0.0" );
  }

  private void writeLocation( PrintWriter outlf, String provName, 
				LocationStorageEntry lse ) {
	outlf.println( provName+","+lse.time+","+lse.longitude+","+lse.latitude+
					",0.0,0.0,0.0" );
  }

  private static void usage() {
	System.out.println( "Usage: java gfxtotrack [-s<speedfactor>] <gfx file> <provname>" );
	System.out.println( "  where <provname> is the name of the provider (<provname>_location and <provname>_track will be generated)" );
    System.out.println( "  <speedfactor> is used to divide time period lengths among the fixes, 1 by default" );
  }

  class LocationStorageEntry {
	public String longitude = null;
	public String latitude = null;
	public long time = -1L;

	public boolean validate() {
	  return ( longitude != null ) &&
			 ( latitude != null ) &&
			 ( time >= 0L );
	}

    public String toString() {
	  StringBuffer sb = new StringBuffer();
	  sb.append( "longitude: "+longitude );
	  sb.append( ",latitude: "+latitude );
	  sb.append( ",time: "+time );
	  return new String( sb );
	}
  }

}
